/////////////////////////////////////////////////////////////////////////////
// Name:        wx/generic/logg.h
// Purpose:     Assorted wxLogXXX functions, and wxLog (sink for logs)
// Author:      Vadim Zeitlin
// Modified by: Christian Sturmlechner, 2005-05-26 - 2019-02-15
// Created:     29/01/98
// RCS-ID:      $Id: logg.h 41020 2006-09-05 20:47:48Z VZ $
// Copyright:   (c) 1998 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

#ifndef   _WX_LOGG_H_
#define   _WX_LOGG_H_

/* old code start -------------------------------------------------------------------------------------------
#if wxUSE_GUI
-- old code end / new code start (Modification by Christian Sturmlechner - STURMLECHNER_HAIKU) ------------ */
#if wxUSE_GUI || defined( __MYBEOS__ )
// new code end ---------------------------------------------------------------------------------------------

// ----------------------------------------------------------------------------
// the following log targets are only compiled in if the we're compiling the
// GUI part (andnot just the base one) of the library, they're implemented in
// src/generic/logg.cpp *and not src/common/log.cpp unlike all the rest)
// ----------------------------------------------------------------------------

#if wxUSE_TEXTCTRL

// log everything to a text window (GUI only of course)
class WXDLLEXPORT wxLogTextCtrl : public wxLog
{
public:
    wxLogTextCtrl(wxTextCtrl *pTextCtrl);

protected:
    // implement sink function
    virtual void DoLogString(const wxChar *szString, time_t t);

private:
    // the control we use
    wxTextCtrl *m_pTextCtrl;

    DECLARE_NO_COPY_CLASS(wxLogTextCtrl)
};

#endif // wxUSE_TEXTCTRL

// ----------------------------------------------------------------------------
// GUI log target, the default one for wxWidgets programs
// ----------------------------------------------------------------------------

#if wxUSE_LOGGUI

class WXDLLEXPORT wxLogGui : public wxLog
{
public:
    // ctor
    wxLogGui();

    // show all messages that were logged since the last Flush()
    virtual void Flush();

protected:
    virtual void DoLog(wxLogLevel level, const wxChar *szString, time_t t);

    // empty everything
    void Clear();

    wxArrayString m_aMessages;      // the log message texts
    wxArrayInt    m_aSeverity;      // one of wxLOG_XXX values
    wxArrayLong   m_aTimes;         // the time of each message
    bool          m_bErrors,        // do we have any errors?
                  m_bWarnings,      // any warnings?
                  m_bHasMessages;   // any messages at all?

};

#endif // wxUSE_LOGGUI

// ----------------------------------------------------------------------------
// (background) log window: this class forwards all log messages to the log
// target which was active when it was instantiated, but also collects them
// to the log window. This window has it's own menu which allows the user to
// close it, clear the log contents or save it to the file.
// ----------------------------------------------------------------------------

#if wxUSE_LOGWINDOW

// new code begin -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner - copied this from log.cpp to logg.h to be able to subclass this
#if wxUSE_SUBCLASSEDLOG

#include "wx/frame.h"
class wxLogWindow;

// log frame class
// ---------------
class wxLogFrame : public wxFrame
{
public:
    // ctor & dtor
    wxLogFrame(wxWindow *pParent, wxLogWindow *log, const wxChar *szTitle
               #if wxUSE_LOGFRAMEEXTRAPARAMETER
               , long _styleoftextctrl = -1
               #endif // wxUSE_LOGFRAMEEXTRAPARAMETER
              );
    virtual ~wxLogFrame();

    // menu callbacks
    void OnClose(wxCommandEvent& event);
    void OnCloseWindow(wxCloseEvent& event);
#if wxUSE_FILE
    void OnSave (wxCommandEvent& event);
#endif // wxUSE_FILE
    void OnClear(wxCommandEvent& event);

    // accessors
    wxTextCtrl *TextCtrl() const { return m_pTextCtrl; }

private:
    // use standard ids for our commands!
    enum
    {
/* old code start -------------------------------------------------------------------------------------------
        Menu_Close = wxID_CLOSE,
        Menu_Save  = wxID_SAVE,
        Menu_Clear = wxID_CLEAR
-- old code end / new code start ------------------------------------------------------------------------- */
// Modification by Christian Sturmlechner STURMLECHNER_F
        Menu_Close = wxID_CLOSELOG,
        Menu_Save  = wxID_SAVELOG,
        Menu_Clear = wxID_CLEARLOG
// new code end ---------------------------------------------------------------------------------------------
    };

    // common part of OnClose() and OnCloseWindow()
/* old code start -------------------------------------------------------------------------------------------
    void DoClose();
-- old code end / new code start ------------------------------------------------------------------------- */
// Modification by Christian Sturmlechner - this is needed for derived classes
protected:
      virtual void DoClose();
private:
// new code end ---------------------------------------------------------------------------------------------

    wxTextCtrl  *m_pTextCtrl;
    wxLogWindow *m_log;

    DECLARE_EVENT_TABLE()
    DECLARE_NO_COPY_CLASS(wxLogFrame)
};

#endif // wxUSE_SUBCLASSEDLOG
// -- new code end ------------------------------------------------------------------------------------------


class WXDLLEXPORT wxLogWindow : public wxLogPassThrough
{
public:
    wxLogWindow(wxWindow *pParent,         // the parent frame (can be NULL)
                const wxChar *szTitle,    // the title of the frame
                bool bShow = true,        // show window immediately?
                bool bPassToOld = true);  // pass messages to the old target?

    virtual ~wxLogWindow();

    // window operations
        // show/hide the log window
    void Show(bool bShow = true);
        // retrieve the pointer to the frame
    wxFrame *GetFrame() const;

    // overridables
        // called immediately after the log frame creation allowing for
        // any extra initializations
    virtual void OnFrameCreate(wxFrame *frame);
        // called if the user closes the window interactively, will not be
        // called if it is destroyed for another reason (such as when program
        // exits) - return true from here to allow the frame to close, false
        // to prevent this from happening
    virtual bool OnFrameClose(wxFrame *frame);
        // called right before the log frame is going to be deleted: will
        // always be called unlike OnFrameClose()
    virtual void OnFrameDelete(wxFrame *frame);

protected:
    virtual void DoLog(wxLogLevel level, const wxChar *szString, time_t t);
    virtual void DoLogString(const wxChar *szString, time_t t);

/* old code begin -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner - this is needed for derived classes ( wxUSE_SUBCLASSEDLOG )
private:
-- old code end ------------------------------------------------------------------------------------------ */
    wxLogFrame *m_pLogFrame;      // the log frame

    DECLARE_NO_COPY_CLASS(wxLogWindow)
};

#endif // wxUSE_LOGWINDOW

#endif // wxUSE_GUI

#endif  // _WX_LOGG_H_

