/////////////////////////////////////////////////////////////////////////////
// Name:        wx/mysetup.h
// Purpose:     additional Configuration for the library
// Author:      Christian Sturmlechner 2005-06-01 - 2024-05-10
// Copyright:   (c) Christian Sturmlechner
// Licence:     wxWindows license
/////////////////////////////////////////////////////////////////////////////

#ifndef _WX_MYSETUP_H_
#define _WX_MYSETUP_H_

#include "wx/setup.h"

// copied from wx-3.2.1
#if defined(_MSVC_LANG)
    #define wxCHECK_CXX_STD(ver) (_MSVC_LANG >= (ver))
#elif defined(__cplusplus)
    #define wxCHECK_CXX_STD(ver) (__cplusplus >= (ver))
#else
    #define wxCHECK_CXX_STD(ver) 0
#endif

#if wxCHECK_CXX_STD(201103L)
    #ifndef HAVE_STRTOULL
        #define HAVE_STRTOULL
    #endif
    #ifndef HAVE_WCSTOULL
        #define HAVE_WCSTOULL
    #endif
#endif

// all changes can be found with a search for Sturmlechner, some changes
// have additional markers:
// my changes to avoid warnings:                  search for STURMLECHNER_W
// my changes to avoid bugs:                      search for STURMLECHNER_B
// my changes due to features in my application:  search for STURMLECHNER_F
// my changes to be able to use an older sdk:     search for STURMLECHNER_SDK
// my changes to use more files with wx-base      search for STURMLECHNER_BASE
// my changes to use more files with haiku        search for STURMLECHNER_HAIKU

// Please keep in mind that this is a work in progress and some changes, esp.
// those for STURMLECHNER_BASE and STURMLECHNER_HAIKU, are only useful for
// special purposes.

// comment out warnings and depricated functions ------------------------------

// set to 0 to avoide messages for errors that are acceptable
#define __WXSTRICTDEBUG__  0
// set to 1 for backwards compatibility of grid
#define wxGRID_OLDFUNCTIONS  0

// my additions to be able to comment out my changes in wx --------------------

// set to 1 if use show messages only useful for TotoCalculator 2
#define wxUSE_COMPILE_FOR_TOTOCALCULATOR 1

#define wxUSE_PAPERFORMATS_FOR_TOTOCALCULATOR 1

// set to 1 if wxLogFrame and wxLogWindow are subclassed
#define wxUSE_SUBCLASSEDLOG 1
// set to 1, if the menubar of the frame is provided by the subclassed frame
// or there should be no menubar.
#if defined( __WXMAC__ )
  #define wxUSE_SUBCLASSEDLOG_EX 1
#else
  #define wxUSE_SUBCLASSEDLOG_EX 0
#endif

#if wxUSE_SUBCLASSEDLOG_EX && !( wxUSE_SUBCLASSEDLOG )
#error wxUSE_SUBCLASSEDLOG_EX needs wxUSE_SUBCLASSEDLOG
#endif

// include wxPreviewCanvas::SetPrintPreview and wxPreviewControlBar::SetPrintPreview
#define wxUSE_PRINTPREVIEW_EX 1

// make wxGrid::SaveEditControlValue() and DrawCell() virtual
#define wxUSE_GRIDEX 1
// workaround for Watcom - this avoids a crash during compilation
#if defined( __WATCOMC__ ) && wxUSE_CHECKBOX
  #define wxUSE_GRID_WORKAROUND_FOR_WATCOM 1
#else
  #define wxUSE_GRID_WORKAROUND_FOR_WATCOM 0
#endif

// include the Neutralize() into the docview architecture
#if defined( __WXMAC__ )
  #define wxUSE_NEUTRALIZE  1
#else
  #define wxUSE_NEUTRALIZE  0 // not implemented
#endif

// wxMac with wx-2.8.8 (and maybe other): If the application is closed by the
// OS (especially if the user closes it via the Dock) if a modal dialogs was
// open then
// 1. some dialogs stay alive, like wxFileSelectorEx
// 2. some dialogs are closed
// In both cases a program tends to crash if the code after the dialog is
// executed (e.g. because the user chooses a file in the file selector).
// After MyFrame::OnCloseWindow was called this function returns true, so
// we can check if we should return after the dialog.
#define wxUSE_MAINFRAMEWASCLOSED 1

// adapt wxStreamBase::Reset()
#define wxUSE_STREAMBASE_EX 1

// has getzlibversion()
#define wxUSE_HASZLIBVERSION 1

// if __WXMAC__ change shortcut of redo to Ctrl+Shift+Z in
// wxCommandProcessor::wxCommandProcessor
#define wxUSE_REDOSHORTCUT 1

// in wxDocManager::CreateDocument(), if wxDOC_SILENT, a new document with the
// first template is silently created
#define wxUSE_WXDOC_SILENT 1

// wxFrame and wxWindow have SetAllCursor()
#define wxUSE_SETALLCURSOR 1

// wxLogFrame has additional parameter for style of textcontrol
#define wxUSE_LOGFRAMEEXTRAPARAMETER 1

// additions in helpview-frame to support subclass
#define wxUSE_SUBCLASSHELPFRAME 1

// msw: avoid iconizing of child frames (just as with mdi child frames)
#define wxUSE_AVOIDICONIZING_OF_CHILD_FRAMES 1

// with 1, the selected file extension is added to the user's input
#if defined( __WXMAC__ ) || defined( __WXGTK__ ) || defined( __MYBEOS__ )
  #define wxUSE_ADAPTEDFILEDLG 1
#else // not needed for msw
  #define wxUSE_ADAPTEDFILEDLG 0
#endif

#define wxUSE_singlechoicedialoghasok 1
// singlechoicedialog: use radiobox if only few items
#define wxUSE_singlechoicedialog_ex 1
#define wxSINGLECHOICEDIALOG_MARGINFORRADIOBOX 10

// additional feature for wxAnimationCtrl
#define wxUSE_AnimationCtrl_Ex 1

#define wxUSE_translationinfo 1 // needed for Translator-version

#if defined( __WXMAC__ )
  #define wxUSE_FD_FILE_PACKAGES_AS_DIRECTORIES 1
#else // !
  #define wxUSE_FD_FILE_PACKAGES_AS_DIRECTORIES 0
#endif // __WXMAC__ / !__WXMAC__

#define wxUSE_PrintfV_sizable 1

#if defined( __WXGTK__ )
  #define wxUSE_GetToolkitVersioneEx 1
#else
  #define wxUSE_GetToolkitVersioneEx 0
#endif

#if defined( __WXMSW__ )
  #define wxUSE_CHOICEHAS_ISVISIBLE       1
  #define wxUSE_WINHAS_ISMENUOPEN         1
  #define wxUSE_WINHAS_HINDERESCAPECLOSE  1
#else // !msw
  #define wxUSE_CHOICEHAS_ISVISIBLE       0
  #define wxUSE_WINHAS_ISMENUOPEN         0
  #define wxUSE_WINHAS_HINDERESCAPECLOSE  0
#endif // msw / !msw

#if defined( __WXGTK20__ )
  // the flags wxALIGN_CENTRE and wxALIGN_RIGHT do not have any effect with some version
  // of GTK+-2.
  // wxUSE_HACKFORSTATTEXTPOSITION works by positioning the control itself (but
  // only implements this where it is needed for TotoCalculator)
  #define wxUSE_HACKFORSTATTEXTPOSITION 1
#else
  #define wxUSE_HACKFORSTATTEXTPOSITION 0
#endif

#define wxUSE_HASMENUITEMSHOWIMAGE 0
#if defined( __WXGTK20__ )
  #include <gtk/gtkversion.h>
  #if GTK_CHECK_VERSION(2, 16, 0)
  // enables to overide the global setting in gtk-2 to determine if the images should be shown in
  // the menus at all. this uses gtk_image_menu_item_set_always_show_image() that is available
  // with gtk+-2.16
  #undef wxUSE_HASMENUITEMSHOWIMAGE
  #define wxUSE_HASMENUITEMSHOWIMAGE 1
  #endif
#endif

#if defined( __WXMAC__ )
  // wxWindowMac and wxToolBar have MacSetStatusBarHelp()
  #define wxUSE_TOOLBAR_HAS_MACSETSTATUSBARHELP 1
  // workaround to get a translated help menu
  #define wxUSE_MAC_HELPMENUFIX 1
#else
  #define wxUSE_TOOLBAR_HAS_MACSETSTATUSBARHELP 0
  #define wxUSE_MAC_HELPMENUFIX 0
#endif

// In wx/defs.h, 'new' is undefined if !wxUSE_IOSTREAMH, this can be changed here
// This is only used for some tests and otherwise should really be set to 0
#if defined( __WXDEBUG__ )
  #define wxUSE_FORCE_DEBUGNEW 0
#else
  #define wxUSE_FORCE_DEBUGNEW 0
#endif
#if ( defined __GNUC__ && __GNUC__ < 3 ) || ( defined __GNUG__ && __GNUG__ < 3 )
  #undef wxUSE_FORCE_DEBUGNEW
  #define wxUSE_FORCE_DEBUGNEW 0
#endif // old gcc-compiler
/*
#if SIZEOF_VOID_P >= 8
  #undef wxUSE_FORCE_DEBUGNEW
  #define wxUSE_FORCE_DEBUGNEW 0
#endif
*/
/* not needed if in config.gcc: USE_EXCEPTIONS ?= 0
#if defined( __WXMSW__ ) && defined( __GNUC__ )
  #undef wxUSE_GLOBAL_MEMORY_OPERATORS
  #define wxUSE_GLOBAL_MEMORY_OPERATORS 0
#endif // msw mit mingw
*/

#if defined( __BEOS__ ) || defined( __UNIX__ )
  #define wxUSE_DebugContextOutputDumpLineTOCONSOLE 1
#else
  #define wxUSE_DebugContextOutputDumpLineTOCONSOLE 0
#endif

#undef wxUSE_UIACTIONSIMULATOR
#if ( defined( __WXMAC__ ) && ( defined(_M_MPPC) || defined(__PPC__) || defined(__ppc__) ) ) || defined( __MYCONSOLE__ ) || defined( __MYBEOS__ ) || defined( __WXX11__ ) || defined( __WXUNIVERSAL__ )
  #define wxUSE_UIACTIONSIMULATOR 0
#else
  #define wxUSE_UIACTIONSIMULATOR 1
#endif

#if defined( __WXGTK__ )
  // This avoids errors if user clicks the button permanently
  #define wxUSE_HACK_SPINBUTT 1
#else
  #define wxUSE_HACK_SPINBUTT 0
#endif

// modifications of wx-definitions --------------------------------------------

// to avoid warning
#if !defined( wxUSE_OWNER_DRAWN )
  #define wxUSE_OWNER_DRAWN 0
#endif

// avoid error with wrong definition

// UNICODE muss hier nicht definiert sein, statt dessen hier wxUSE_UNICODE
#if wxUSE_UNICODE && ( defined( __WXMAC__ ) || defined( __FREEBSD__ ) )
  #if HAVE_UNIX98_PRINTF
    #undef HAVE_UNIX98_PRINTF
    #define HAVE_UNIX98_PRINTF 0
  #endif
#endif

// build-specific settings ----------------------------------------------------

// ----------------------------------------------------------------------------
// Mac-only settings
// ----------------------------------------------------------------------------

// only needed for mac and defined in src/mac/carbon/bitmap.cpp
// avoid memory leaks with bitmaps in menuitems
// #define wxUSE_DELETE_BITMAP_DATAPROVIDER 1

#if defined( __WXMAC__ )
  // currently not used #include "wx/mybuildsetup.h"

  #define wxMAC_USE_NATIVE_TOOLBAR 0

  // this sets wxUSE_translationinfo to 0 if it is not needed
  #include "wx/mysetup2.h"

#endif // __WXMAC__

#if !wxUSE_GUI && !defined( __WXGTK__ ) && !defined( __WXMAC__ ) && !defined( __WXMSW__ ) && !defined( __WINDOWS__ )
  #if defined( __WINDOWS__ ) || defined( __WXMSW__ )
    // use mysetup_base_win.h
  #elif defined( __BEOS__ )
    // use #include <../../include/wx/mysetup_haiku.h>
  #else
    #include <wx/mysetup_base.h>
  #endif
#endif

#endif // _WX_MYSETUP_H_
