/////////////////////////////////////////////////////////////////////////////
// Name:        src/common/fldlgcmn.cpp
// Purpose:     wxFileDialog common functions
// Author:      John Labenski
// Modified by: Christian Sturmlechner 2005/03/12-2011/02/20
// Created:     14.06.03 (extracted from src/*/filedlg.cpp)
// RCS-ID:      $Id: fldlgcmn.cpp 66916 2011-02-16 21:48:55Z JS $
// Copyright:   (c) Robert Roebling
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

#ifdef __BORLANDC__
#pragma hdrstop
#endif

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#if wxUSE_FILEDLG

#include "wx/filedlg.h"
#include "wx/dirdlg.h"

#ifndef WX_PRECOMP
    #include "wx/string.h"
    #include "wx/intl.h"
    #include "wx/window.h"
#endif // WX_PRECOMP

// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#if wxUSE_ADAPTEDFILEDLG
#include "wx/filename.h"    // wxFileName
#include "wx/app.h"         // wxTheApp
#include "wx/msgdlg.h"      // wxMessageBox
#endif // wxUSE_ADAPTEDFILEDLG
#if wxUSE_MAINFRAMEWASCLOSED
#include "wx/frame.h"       // wxFrameBase
#endif // wxUSE_MAINFRAMEWASCLOSED
// new code end ---------------------------------------------------------------------------------------------

//----------------------------------------------------------------------------
// wxFileDialogBase
//----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxFileDialogBase, wxDialog)

void wxFileDialogBase::Init()
{
    m_filterIndex =
    m_windowStyle = 0;
}

bool wxFileDialogBase::Create(wxWindow *parent,
                              const wxString& message,
                              const wxString& defaultDir,
                              const wxString& defaultFile,
                              const wxString& wildCard,
                              long style,
                              const wxPoint& WXUNUSED(pos),
                              const wxSize& WXUNUSED(sz),
                              const wxString& WXUNUSED(name))
{
    m_message = message;
    m_dir = defaultDir;
    m_fileName = defaultFile;
    m_wildCard = wildCard;

    m_parent = parent;
    m_windowStyle = style;
    m_filterIndex = 0;

    if (!HasFdFlag(wxFD_OPEN) && !HasFdFlag(wxFD_SAVE))
        m_windowStyle |= wxFD_OPEN;     // wxFD_OPEN is the default

    // check that the styles are not contradictory
    wxASSERT_MSG( !(HasFdFlag(wxFD_SAVE) && HasFdFlag(wxFD_OPEN)),
                  _T("can't specify both wxFD_SAVE and wxFD_OPEN at once") );

    wxASSERT_MSG( !HasFdFlag(wxFD_SAVE) ||
                    (!HasFdFlag(wxFD_MULTIPLE) && !HasFdFlag(wxFD_FILE_MUST_EXIST)),
                   _T("wxFD_MULTIPLE or wxFD_FILE_MUST_EXIST can't be used with wxFD_SAVE" ) );

    wxASSERT_MSG( !HasFdFlag(wxFD_OPEN) || !HasFdFlag(wxFD_OVERWRITE_PROMPT),
                  _T("wxFD_OVERWRITE_PROMPT can't be used with wxFD_OPEN") );

    if ( wildCard.empty() || wildCard == wxFileSelectorDefaultWildcardStr )
    {
        m_wildCard = wxString::Format(_("All files (%s)|%s"),
                                      wxFileSelectorDefaultWildcardStr,
                                      wxFileSelectorDefaultWildcardStr);
    }
    else // have wild card
    {
        // convert m_wildCard from "*.bar" to "bar files (*.bar)|*.bar"
        if ( m_wildCard.Find(wxT('|')) == wxNOT_FOUND )
        {
            wxString::size_type nDot = m_wildCard.find(_T("*."));
            if ( nDot != wxString::npos )
                nDot++;
            else
                nDot = 0;

            m_wildCard = wxString::Format
                         (
                            _("%s files (%s)|%s"),
                            wildCard.c_str() + nDot,
                            wildCard.c_str(),
                            wildCard.c_str()
                         );
        }
    }

    return true;
}

#if WXWIN_COMPATIBILITY_2_4
// Parses the filterStr, returning the number of filters.
// Returns 0 if none or if there's a problem.
// filterStr is in the form: "All files (*.*)|*.*|JPEG Files (*.jpeg)|*.jpg"
int wxFileDialogBase::ParseWildcard(const wxString& filterStr,
                                    wxArrayString& descriptions,
                                    wxArrayString& filters)
{
    return ::wxParseCommonDialogsFilter(filterStr, descriptions, filters);
}
#endif // WXWIN_COMPATIBILITY_2_4

#if WXWIN_COMPATIBILITY_2_6
long wxFileDialogBase::GetStyle() const
{
    return GetWindowStyle();
}

void wxFileDialogBase::SetStyle(long style)
{
    SetWindowStyle(style);
}
#endif // WXWIN_COMPATIBILITY_2_6


wxString wxFileDialogBase::AppendExtension(const wxString &filePath,
                                           const wxString &extensionList)
{
    // strip off path, to avoid problems with "path.bar/foo"
    wxString fileName = filePath.AfterLast(wxFILE_SEP_PATH);

    // if fileName is of form "foo.bar" it's ok, return it
    int idx_dot = fileName.Find(wxT('.'), true);
    if ((idx_dot != wxNOT_FOUND) && (idx_dot < (int)fileName.length() - 1))
        return filePath;

    // get the first extension from extensionList, or all of it
    wxString ext = extensionList.BeforeFirst(wxT(';'));

    // if ext == "foo" or "foo." there's no extension
    int idx_ext_dot = ext.Find(wxT('.'), true);
    if ((idx_ext_dot == wxNOT_FOUND) || (idx_ext_dot == (int)ext.length() - 1))
        return filePath;
    else
        ext = ext.AfterLast(wxT('.'));

    // if ext == "*" or "bar*" or "b?r" or " " then its not valid
    if ((ext.Find(wxT('*')) != wxNOT_FOUND) ||
        (ext.Find(wxT('?')) != wxNOT_FOUND) ||
        (ext.Strip(wxString::both).empty()))
        return filePath;

    // if fileName doesn't have a '.' then add one
    if (filePath.Last() != wxT('.'))
        ext = wxT(".") + ext;

    return filePath + ext;
}

//----------------------------------------------------------------------------
// wxFileDialog convenience functions
//----------------------------------------------------------------------------

wxString wxFileSelector(const wxChar *title,
                               const wxChar *defaultDir,
                               const wxChar *defaultFileName,
                               const wxChar *defaultExtension,
                               const wxChar *filter,
                               int flags,
                               wxWindow *parent,
/* old code start -------------------------------------------------------------------------------------------
                               int x, int y)
-- old code end / new code start ------------------------------------------------------------------------- */
// Modification by Christian Sturmlechner
                               int x, int y
                                                    #if wxUSE_ADAPTEDFILEDLG
                                                    , int firstfilterindex
                                                    , int *selectedindex
                                                    #endif // wxUSE_ADAPTEDFILEDLG
                                                                           )
// new code end ---------------------------------------------------------------------------------------------
{
    // The defaultExtension, if non-NULL, is
    // appended to the filename if the user fails to type an extension. The new
    // implementation (taken from wxFileSelectorEx) appends the extension
    // automatically, by looking at the filter specification. In fact this
    // should be better than the native Microsoft implementation because
    // Windows only allows *one* default extension, whereas here we do the
    // right thing depending on the filter the user has chosen.

    // If there's a default extension specified but no filter, we create a
    // suitable filter.

    wxString filter2;
    if ( !wxIsEmpty(defaultExtension) && wxIsEmpty(filter) )
        filter2 = wxString(wxT("*.")) + defaultExtension;
    else if ( !wxIsEmpty(filter) )
        filter2 = filter;

    wxString defaultDirString;
    if (!wxIsEmpty(defaultDir))
        defaultDirString = defaultDir;

    wxString defaultFilenameString;
    if (!wxIsEmpty(defaultFileName))
        defaultFilenameString = defaultFileName;

// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#if !wxUSE_ADAPTEDFILEDLG
// new code end ---------------------------------------------------------------------------------------------
    wxFileDialog fileDialog(parent, title, defaultDirString,
                            defaultFilenameString, filter2,
                            flags, wxPoint(x, y));

   // if filter is of form "All files (*)|*|..." set correct filter index
    if((wxStrlen(defaultExtension) != 0) && (filter2.Find(wxT('|')) != wxNOT_FOUND))
    {
        int filterIndex = 0;

        wxArrayString descriptions, filters;
        // don't care about errors, handled already by wxFileDialog
        (void)wxParseCommonDialogsFilter(filter2, descriptions, filters);
        for (size_t n=0; n<filters.GetCount(); n++)
        {
            if (filters[n].Contains(defaultExtension))
            {
                filterIndex = n;
                        break;
            }
        }

        if (filterIndex > 0)
            fileDialog.SetFilterIndex(filterIndex);
    }

    wxString filename;
/* old code start -------------------------------------------------------------------------------------------
    if ( fileDialog.ShowModal() == wxID_OK )
-- old code end / new code start ------------------------------------------------------------------------- */
// Modification by Christian Sturmlechner
    int iRet = fileDialog.ShowModal();
    #if wxUSE_MAINFRAMEWASCLOSED
    if ( wxFrameBase::MainFrameWasClosed() ) { return wxGetEmptyString(); }
    #endif // wxUSE_MAINFRAMEWASCLOSED
    if ( iRet == wxID_OK )
// new code end ---------------------------------------------------------------------------------------------
    {
        filename = fileDialog.GetPath();
    }

    return filename;
// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner added a check if the file name with the extension exists
#else // wxUSE_ADAPTEDFILEDLG
    int filterindex = firstfilterindex;
    wxArrayString descriptions, filters;
    if( filter2.Find(wxT('|')) != wxNOT_FOUND )
    {
        // don't care about errors, handled already by wxFileDialog
        wxParseCommonDialogsFilter(filter2, descriptions, filters);
    }
    while(1)
    {
        // wxOVERWRITE_PROMPT is removed from the flags - we check ourselves
        wxFileDialog fileDialog(parent, title, defaultDirString,
                                defaultFilenameString, filter2,
                                flags & ~wxFD_OVERWRITE_PROMPT, wxPoint(x, y));
        fileDialog.SetFilterIndex( filterindex );
        int ret = fileDialog.ShowModal();
// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
        #if wxUSE_MAINFRAMEWASCLOSED
        if ( wxFrameBase::MainFrameWasClosed() ) { return wxGetEmptyString(); }
        #endif // wxUSE_MAINFRAMEWASCLOSED
// new code end ---------------------------------------------------------------------------------------------
        if ( ret == wxID_OK )
        {
            wxString fileNameAndPath( fileDialog.GetPath() );
            filterindex = fileDialog.GetFilterIndex();
            if ( selectedindex )
            {
                *selectedindex = filterindex;
            }
            if (    ( flags & wxFD_SAVE )
                 && ( flags & wxFD_OVERWRITE_PROMPT )
               )
            {
                wxString userExtension, defaultExtensionToUse;
                wxFileName::SplitPath( fileNameAndPath, &defaultDirString, &defaultFilenameString,
                                       &userExtension );
                // create the default-extension
                if ( filters.GetCount() > 0 )
                {
                    wxFileName::SplitPath( filters[filterindex], NULL, NULL, &defaultExtensionToUse );
                }
                else
                {
                    defaultExtensionToUse.assign( defaultExtension );
                }
                // the extension entered by the user shall not be losed (it might be simply a point
                // in the file name and not be considerer as begin of the extension
                if ( !userExtension.IsEmpty() )
                {
                    defaultFilenameString.append( wxT(".") );
                    defaultFilenameString.append( userExtension );
                }

                // append the selected extension if there is none or if the user entered an
                // extension other than the default extension
                if (    userExtension.IsEmpty()
                     || userExtension != defaultExtensionToUse
                   )
                {
                    if ( !defaultExtensionToUse.empty() ) // this is the case if defaultExtensionToUse is empty
                    {
                        fileNameAndPath += wxT(".");
                        fileNameAndPath += defaultExtensionToUse;
                        defaultFilenameString += wxT(".");
                        defaultFilenameString += defaultExtensionToUse;
                    }
                }

                if ( wxFileExists( fileNameAndPath ) )
                {
                    wxString msgTitle;
                    if ( wxTheApp->GetAppName() != wxT("") )
                    {
                        msgTitle.assign( wxTheApp->GetAppName() );
                    }
                    else
                    {
                        msgTitle.assign( _("Warning") );
                    }

                    wxString prompt;
                    prompt.Printf(_("File '%s' already exists, do you really want to overwrite it?"),
                            defaultFilenameString.c_str() );
                    int res = wxMessageBox( prompt, msgTitle,
                                            wxYES_NO|wxCANCEL|wxICON_QUESTION,
                                            parent );
                    if ( res == wxNO )
                    {
                        continue;
                    }
                    else if ( res == wxYES )
                    {
                        return fileNameAndPath;
                    }
                    else if ( res == wxCANCEL )
                    {
                        return wxGetEmptyString();
                    }
                }
            }
            return fileNameAndPath;
        }
        else
        {
            return wxGetEmptyString();
        }
    }
#endif // !wxUSE_ADAPTEDFILEDLG / wxUSE_ADAPTEDFILEDLG
// new code end ---------------------------------------------------------------------------------------------
}

//----------------------------------------------------------------------------
// wxFileSelectorEx
//----------------------------------------------------------------------------

wxString wxFileSelectorEx(const wxChar *title,
                          const wxChar *defaultDir,
                          const wxChar *defaultFileName,
                          int* defaultFilterIndex,
                          const wxChar *filter,
                          int       flags,
                          wxWindow* parent,
                          int       x,
                          int       y)

{
    wxFileDialog fileDialog(parent,
                            !wxIsEmpty(title) ? title : wxEmptyString,
                            !wxIsEmpty(defaultDir) ? defaultDir : wxEmptyString,
                            !wxIsEmpty(defaultFileName) ? defaultFileName : wxEmptyString,
                            !wxIsEmpty(filter) ? filter : wxEmptyString,
                            flags, wxPoint(x, y));

    wxString filename;
/* old code start -------------------------------------------------------------------------------------------
    if ( fileDialog.ShowModal() == wxID_OK )
-- old code end / new code start ------------------------------------------------------------------------- */
// Modification by Christian Sturmlechner
    int ret = fileDialog.ShowModal();
    #if wxUSE_MAINFRAMEWASCLOSED
    if ( wxFrameBase::MainFrameWasClosed() ) { return wxGetEmptyString(); }
    #endif // wxUSE_MAINFRAMEWASCLOSED
    if ( ret == wxID_OK )
// new code end ---------------------------------------------------------------------------------------------
    {
        if ( defaultFilterIndex )
            *defaultFilterIndex = fileDialog.GetFilterIndex();

        filename = fileDialog.GetPath();
    }

    return filename;
}

//----------------------------------------------------------------------------
// wxDefaultFileSelector - Generic load/save dialog (for internal use only)
//----------------------------------------------------------------------------

static wxString wxDefaultFileSelector(bool load,
                                      const wxChar *what,
                                      const wxChar *extension,
                                      const wxChar *default_name,
                                      wxWindow *parent)
{
    wxString prompt;
    wxString str;
    if (load)
        str = _("Load %s file");
    else
        str = _("Save %s file");
    prompt.Printf(str, what);

    wxString wild;
    const wxChar *ext = extension;
    if ( !wxIsEmpty(ext) )
    {
        if ( *ext == wxT('.') )
            ext++;

        wild.Printf(wxT("*.%s"), ext);
    }
    else // no extension specified
    {
        wild = wxFileSelectorDefaultWildcardStr;
    }

    return wxFileSelector(prompt, NULL, default_name, ext, wild,
                          load ? (wxFD_OPEN | wxFD_FILE_MUST_EXIST) : wxFD_SAVE, parent);
}

//----------------------------------------------------------------------------
// wxLoadFileSelector
//----------------------------------------------------------------------------

WXDLLEXPORT wxString wxLoadFileSelector(const wxChar *what,
                                        const wxChar *extension,
                                        const wxChar *default_name,
                                        wxWindow *parent)
{
    return wxDefaultFileSelector(true, what, extension, default_name, parent);
}

//----------------------------------------------------------------------------
// wxSaveFileSelector
//----------------------------------------------------------------------------

WXDLLEXPORT wxString wxSaveFileSelector(const wxChar *what,
                                        const wxChar *extension,
                                        const wxChar *default_name,
                                        wxWindow *parent)
{
    return wxDefaultFileSelector(false, what, extension, default_name, parent);
}


//----------------------------------------------------------------------------
// wxDirDialogBase
//----------------------------------------------------------------------------

#if WXWIN_COMPATIBILITY_2_6
long wxDirDialogBase::GetStyle() const
{
    return GetWindowStyle();
}

void wxDirDialogBase::SetStyle(long style)
{
    SetWindowStyle(style);
}
#endif // WXWIN_COMPATIBILITY_2_6


#endif // wxUSE_FILEDLG
