/////////////////////////////////////////////////////////////////////////////
// Name:        src/generic/choicdgg.cpp
// Purpose:     Choice dialogs
// Author:      Julian Smart
// Modified by: 03.11.00: VZ to add wxArrayString and multiple sel functions
// Modified by: Christian Sturmlechner 2009/10/25-2013/07/21
// Created:     04/01/98
// RCS-ID:      $Id: choicdgg.cpp 44620 2007-03-06 09:56:43Z JS $
// Copyright:   (c) wxWidgets team
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

// ============================================================================
// declarations
// ============================================================================

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#if wxUSE_CHOICEDLG

#ifndef WX_PRECOMP
    #include <stdio.h>
    #include "wx/utils.h"
    #include "wx/dialog.h"
    #include "wx/button.h"
    #include "wx/listbox.h"
    #include "wx/checklst.h"
    #include "wx/stattext.h"
    #include "wx/intl.h"
    #include "wx/sizer.h"
    #include "wx/arrstr.h"
#endif

#include "wx/statline.h"
#include "wx/settings.h"
#include "wx/generic/choicdgg.h"

// ----------------------------------------------------------------------------
// constants
// ----------------------------------------------------------------------------

#define wxID_LISTBOX 3000

// ----------------------------------------------------------------------------
// private functions
// ----------------------------------------------------------------------------

// convert wxArrayString into a wxString[] which must be delete[]d by caller
static int ConvertWXArrayToC(const wxArrayString& aChoices, wxString **choices);

// ============================================================================
// implementation
// ============================================================================

// ----------------------------------------------------------------------------
// helpers
// ----------------------------------------------------------------------------

int ConvertWXArrayToC(const wxArrayString& aChoices, wxString **choices)
{
    int n = aChoices.GetCount();
    *choices = new wxString[n];

    for ( int i = 0; i < n; i++ )
    {
        (*choices)[i] = aChoices[i];
    }

    return n;
}

// ----------------------------------------------------------------------------
// wrapper functions
// ----------------------------------------------------------------------------

wxString wxGetSingleChoice( const wxString& message,
                            const wxString& caption,
                            int n, const wxString *choices,
                            wxWindow *parent,
                            int WXUNUSED(x), int WXUNUSED(y),
                            bool WXUNUSED(centre),
                            int WXUNUSED(width), int WXUNUSED(height) )
{
    wxSingleChoiceDialog dialog(parent, message, caption, n, choices);
    wxString choice;
    if ( dialog.ShowModal() == wxID_OK )
        choice = dialog.GetStringSelection();

    return choice;
}

wxString wxGetSingleChoice( const wxString& message,
                            const wxString& caption,
                            const wxArrayString& aChoices,
                            wxWindow *parent,
                            int x, int y,
                            bool centre,
                            int width, int height)
{
    wxString *choices;
    int n = ConvertWXArrayToC(aChoices, &choices);
    wxString res = wxGetSingleChoice(message, caption, n, choices, parent,
                                     x, y, centre, width, height);
    delete [] choices;

    return res;
}

int wxGetSingleChoiceIndex( const wxString& message,
                            const wxString& caption,
                            int n, const wxString *choices,
                            wxWindow *parent,
                            int WXUNUSED(x), int WXUNUSED(y),
                            bool WXUNUSED(centre),
                            int WXUNUSED(width), int WXUNUSED(height) )
{
    wxSingleChoiceDialog dialog(parent, message, caption, n, choices);
    int choice;
    if ( dialog.ShowModal() == wxID_OK )
        choice = dialog.GetSelection();
    else
        choice = -1;

    return choice;
}

int wxGetSingleChoiceIndex( const wxString& message,
                            const wxString& caption,
                            const wxArrayString& aChoices,
                            wxWindow *parent,
                            int x, int y,
                            bool centre,
                            int width, int height)
{
    wxString *choices;
    int n = ConvertWXArrayToC(aChoices, &choices);
    int res = wxGetSingleChoiceIndex(message, caption, n, choices, parent,
                                     x, y, centre, width, height);
    delete [] choices;

    return res;
}

void *wxGetSingleChoiceData( const wxString& message,
                             const wxString& caption,
                             int n, const wxString *choices,
                             void **client_data,
                             wxWindow *parent,
                             int WXUNUSED(x), int WXUNUSED(y),
                             bool WXUNUSED(centre),
                             int WXUNUSED(width), int WXUNUSED(height) )
{
/* old code start -------------------------------------------------------------------------------------------
    wxSingleChoiceDialog dialog(parent, message, caption, n, choices,
                                (char **)client_data);
    void *data;
    if ( dialog.ShowModal() == wxID_OK )
        data = dialog.GetSelectionClientData();
    else
        data = NULL;
-- old code end / new code start ------------------------------------------------------------------------- */
// Modification by Christian Sturmlechner
    #if wxUSE_singlechoicedialog_ex
    // TODO caption is recycled for the title of the radiobox - add new definitions in the headers if
    // this is not wanted.
    wxSingleChoiceDialog dialog(parent, message, caption, caption, n, choices,
                                (char **)client_data);
    void *data;
    if ( dialog.ShowModal() == wxID_OK )
    {
        if ( dialog.uselistbox() )
        {
            data = dialog.GetSelectionClientData();
        }
        else // radiobox
        {
            data = client_data[dialog.GetSelection()];
        }
    }
    else
    {
        data = NULL;
    }
    #else // !wxUSE_singlechoicedialog_ex
    wxSingleChoiceDialog dialog(parent, message, caption, n, choices,
                                (char **)client_data);
    void *data;
    if ( dialog.ShowModal() == wxID_OK )
        data = dialog.GetSelectionClientData();
    else
        data = NULL;
    #endif // wxUSE_singlechoicedialog_ex / !wxUSE_singlechoicedialog_ex
// new code end ---------------------------------------------------------------------------------------------

    return data;
}

void *wxGetSingleChoiceData( const wxString& message,
                             const wxString& caption,
                             const wxArrayString& aChoices,
                             void **client_data,
                             wxWindow *parent,
                             int x, int y,
                             bool centre,
                             int width, int height)
{
    wxString *choices;
    int n = ConvertWXArrayToC(aChoices, &choices);
    void *res = wxGetSingleChoiceData(message, caption, n, choices,
                                      client_data, parent,
                                      x, y, centre, width, height);
    delete [] choices;

    return res;
}

size_t wxGetMultipleChoices(wxArrayInt& selections,
                            const wxString& message,
                            const wxString& caption,
                            int n, const wxString *choices,
                            wxWindow *parent,
                            int WXUNUSED(x), int WXUNUSED(y),
                            bool WXUNUSED(centre),
                            int WXUNUSED(width), int WXUNUSED(height))
{
    wxMultiChoiceDialog dialog(parent, message, caption, n, choices);

    // call this even if selections array is empty and this then (correctly)
    // deselects the first item which is selected by default
    dialog.SetSelections(selections);

    if ( dialog.ShowModal() == wxID_OK )
        selections = dialog.GetSelections();
    else
        selections.Empty();

    return selections.GetCount();
}

size_t wxGetMultipleChoices(wxArrayInt& selections,
                            const wxString& message,
                            const wxString& caption,
                            const wxArrayString& aChoices,
                            wxWindow *parent,
                            int x, int y,
                            bool centre,
                            int width, int height)
{
    wxString *choices;
    int n = ConvertWXArrayToC(aChoices, &choices);
    size_t res = wxGetMultipleChoices(selections, message, caption,
                                      n, choices, parent,
                                      x, y, centre, width, height);
    delete [] choices;

    return res;
}

// ----------------------------------------------------------------------------
// wxAnyChoiceDialog
// ----------------------------------------------------------------------------

// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#if wxUSE_singlechoicedialog_ex
bool wxAnyChoiceDialog::Create(wxWindow *parent,
                               const wxString& message,
                               const wxString& caption,
                               int n, const wxString *choices,
                               long styleDlg,
                               const wxPoint& pos,
                               long styleLbox)
{
    return Create( parent, message, caption, wxEmptyString, n, choices, styleDlg, pos, styleLbox );
}
#endif // wxUSE_singlechoicedialog_ex
// new code end ---------------------------------------------------------------------------------------------

bool wxAnyChoiceDialog::Create(wxWindow *parent,
                               const wxString& message,
                               const wxString& caption,
// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
                               #if wxUSE_singlechoicedialog_ex
                               const wxString& title,
                               #endif // wxUSE_singlechoicedialog_ex
// new code end ---------------------------------------------------------------------------------------------
                               int n, const wxString *choices,
                               long styleDlg,
                               const wxPoint& pos,
                               long styleLbox)
{
#ifdef __WXMAC__
    // FIXME: why??
    if ( !wxDialog::Create(parent, wxID_ANY, caption, pos, wxDefaultSize, styleDlg & (~wxCANCEL) ) )
        return false;
#else
    if ( !wxDialog::Create(parent, wxID_ANY, caption, pos, wxDefaultSize, styleDlg) )
        return false;
#endif

    wxBoxSizer *topsizer = new wxBoxSizer( wxVERTICAL );

    // 1) text message
    topsizer->
        Add(CreateTextSizer(message), wxSizerFlags().Expand().TripleBorder());

    // 2) list box
/* old code start -------------------------------------------------------------------------------------------
    m_listbox = CreateList(n, choices, styleLbox);

    if ( n > 0 )
        m_listbox->SetSelection(0);

    topsizer->
        Add(m_listbox, wxSizerFlags().Expand().Proportion(1).TripleBorder(wxLEFT | wxRIGHT));
-- old code end / new code start ------------------------------------------------------------------------- */
// Modification by Christian Sturmlechner
    #if wxUSE_singlechoicedialog_ex
    if (    n > wxSINGLECHOICEDIALOG_MARGINFORRADIOBOX
         || title == wxEmptyString
       )
    {
        m_bUseListBox = true;
        m_listbox = CreateList(n, choices, styleLbox);
        m_radiobox = NULL;

        if ( n > 0 )
            m_listbox->SetSelection(0);

        topsizer->
            Add(m_listbox, wxSizerFlags().Expand().Proportion(1).TripleBorder(wxLEFT | wxRIGHT));
    }
    else
    {
        m_bUseListBox = false;
        #if defined( __WXGTK20__ )
        wxString _title( title );
        _title.Replace( wxT("&"), wxEmptyString );
        #endif // gtk2
        m_listbox = NULL;
        m_radiobox = new wxRadioBox( this, wxID_LISTBOX,
                          #if defined( __WXGTK20__ )
                          _title,
                          #else // !__WXGTK20__
                          title,
                          #endif // __WXGTK20__ / !__WXGTK20__
                          wxDefaultPosition, wxDefaultSize,
                          n, choices, 0, wxRA_SPECIFY_ROWS );

        if ( n > 0 )
            m_radiobox->SetSelection(0);

        topsizer->
            Add(m_radiobox, wxSizerFlags().Expand().Proportion(1).TripleBorder(wxLEFT | wxRIGHT));
    }
    #else // !wxUSE_singlechoicedialog_ex
    m_listbox = CreateList(n, choices, styleLbox);

    if ( n > 0 )
        m_listbox->SetSelection(0);

    topsizer->
        Add(m_listbox, wxSizerFlags().Expand().Proportion(1).TripleBorder(wxLEFT | wxRIGHT));
    #endif // wxUSE_singlechoicedialog_ex / !wxUSE_singlechoicedialog_ex
// new code end ---------------------------------------------------------------------------------------------

    // 3) buttons if any
    wxSizer *
        buttonSizer = CreateSeparatedButtonSizer(styleDlg & ButtonSizerFlags);
    if ( buttonSizer )
    {
        topsizer->Add(buttonSizer, wxSizerFlags().Expand().DoubleBorder());
    }

    SetSizer( topsizer );

    topsizer->SetSizeHints( this );
    topsizer->Fit( this );

    if ( styleDlg & wxCENTRE )
        Centre(wxBOTH);

/* old code start -------------------------------------------------------------------------------------------
    m_listbox->SetFocus();
-- old code end / new code start ------------------------------------------------------------------------- */
// Modification by Christian Sturmlechner
    #if wxUSE_singlechoicedialog_ex
    if ( m_listbox )
    {
        m_listbox->SetFocus();
    }
    else
    {
        m_radiobox->SetFocus();
    }
    #else // !wxUSE_singlechoicedialog_ex
    m_listbox->SetFocus();
    #endif // wxUSE_singlechoicedialog_ex / !wxUSE_singlechoicedialog_ex
// new code end ---------------------------------------------------------------------------------------------

    return true;
}

bool wxAnyChoiceDialog::Create(wxWindow *parent,
                               const wxString& message,
                               const wxString& caption,
                               const wxArrayString& choices,
                               long styleDlg,
                               const wxPoint& pos,
                               long styleLbox)
{
    wxCArrayString chs(choices);
    return Create(parent, message, caption, chs.GetCount(), chs.GetStrings(),
                  styleDlg, pos, styleLbox);
}

wxListBoxBase *wxAnyChoiceDialog::CreateList(int n, const wxString *choices, long styleLbox)
{
	wxSize size = wxDefaultSize;
	if (wxSystemSettings::GetScreenType() > wxSYS_SCREEN_PDA)
		size = wxSize(300, 200);
    return new wxListBox( this, wxID_LISTBOX,
                          wxDefaultPosition, size,
                          n, choices,
                          styleLbox );
}

// ----------------------------------------------------------------------------
// wxSingleChoiceDialog
// ----------------------------------------------------------------------------

BEGIN_EVENT_TABLE(wxSingleChoiceDialog, wxDialog)
    EVT_BUTTON(wxID_OK, wxSingleChoiceDialog::OnOK)
#ifndef __SMARTPHONE__
    EVT_LISTBOX_DCLICK(wxID_LISTBOX, wxSingleChoiceDialog::OnListBoxDClick)
#endif
#ifdef __WXWINCE__
    EVT_JOY_BUTTON_DOWN(wxSingleChoiceDialog::OnJoystickButtonDown)
#endif
END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(wxSingleChoiceDialog, wxDialog)

wxSingleChoiceDialog::wxSingleChoiceDialog(wxWindow *parent,
                                           const wxString& message,
                                           const wxString& caption,
                                           int n,
                                           const wxString *choices,
                                           char **clientData,
                                           long style,
                                           const wxPoint& WXUNUSED(pos))
{
/* old code start -------------------------------------------------------------------------------------------
    Create(parent, message, caption, n, choices, clientData, style);
-- old code end / new code start ------------------------------------------------------------------------- */
// Modification by Christian Sturmlechner
    if ( !Create(parent, message, caption, n, choices, clientData, style) )
    {
      #if wxUSE_singlechoicedialoghasok
      bIsOk = false;
      #endif // wxUSE_singlechoicedialoghasok
      return;
    }
    #if wxUSE_singlechoicedialoghasok
    bIsOk = true;
    #endif // wxUSE_singlechoicedialoghasok
    wxAcceleratorEntry entries[1];
    entries[0].Set(wxACCEL_CTRL, 'W', wxID_CANCEL);
    wxAcceleratorTable accel(1, entries);
    SetAcceleratorTable(accel);
// new code end ---------------------------------------------------------------------------------------------
}

wxSingleChoiceDialog::wxSingleChoiceDialog(wxWindow *parent,
                                           const wxString& message,
                                           const wxString& caption,
                                           const wxArrayString& choices,
                                           char **clientData,
                                           long style,
                                           const wxPoint& WXUNUSED(pos))
{
/* old code start -------------------------------------------------------------------------------------------
    Create(parent, message, caption, choices, clientData, style);
-- old code end / new code start ------------------------------------------------------------------------- */
// Modification by Christian Sturmlechner
    if ( !Create(parent, message, caption, choices, clientData, style) )
    {
      #if wxUSE_singlechoicedialoghasok
      bIsOk = false;
      #endif // wxUSE_singlechoicedialoghasok
      return;
    }
    #if wxUSE_singlechoicedialoghasok
    bIsOk = true;
    #endif // wxUSE_singlechoicedialoghasok
    wxAcceleratorEntry entries[1];
    entries[0].Set(wxACCEL_CTRL, 'W', wxID_CANCEL);
    wxAcceleratorTable accel(1, entries);
    SetAcceleratorTable(accel);
// new code end ---------------------------------------------------------------------------------------------
}

// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#if wxUSE_singlechoicedialog_ex
wxSingleChoiceDialog::wxSingleChoiceDialog(wxWindow *parent,
                                           const wxString& message,
                                           const wxString& caption,
                                           const wxString& title, // <<< for the radiobox
                                           int n,
                                           const wxString *choices,
                                           char **clientData,
                                           long style,
                                           const wxPoint& pos)
{
    if ( !Create(parent, message, caption, title, n, choices, clientData, style, pos) )
    {
      #if wxUSE_singlechoicedialoghasok
      bIsOk = false;
      #endif // wxUSE_singlechoicedialoghasok
      return;
    }
    #if wxUSE_singlechoicedialoghasok
    bIsOk = true;
    #endif // wxUSE_singlechoicedialoghasok
    wxAcceleratorEntry entries[1];
    entries[0].Set(wxACCEL_CTRL, 'W', wxID_CANCEL);
    wxAcceleratorTable accel(1, entries);
    SetAcceleratorTable(accel);
}
#endif // wxUSE_singlechoicedialog_ex
// new code end ---------------------------------------------------------------------------------------------

bool wxSingleChoiceDialog::Create( wxWindow *parent,
                                   const wxString& message,
                                   const wxString& caption,
                                   int n,
                                   const wxString *choices,
                                   char **clientData,
                                   long style,
                                   const wxPoint& pos )
{
    if ( !wxAnyChoiceDialog::Create(parent, message, caption,
                                    n, choices,
                                    style, pos) )
        return false;

    m_selection = n > 0 ? 0 : -1;

// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
    wxASSERT( m_listbox );
// new code end ---------------------------------------------------------------------------------------------
    if (clientData)
    {
        for (int i = 0; i < n; i++)
            m_listbox->SetClientData(i, clientData[i]);
    }

    return true;
}

bool wxSingleChoiceDialog::Create( wxWindow *parent,
                                   const wxString& message,
                                   const wxString& caption,
                                   const wxArrayString& choices,
                                   char **clientData,
                                   long style,
                                   const wxPoint& pos )
{
    wxCArrayString chs(choices);
    return Create( parent, message, caption, chs.GetCount(), chs.GetStrings(),
                   clientData, style, pos );
}

// Set the selection
void wxSingleChoiceDialog::SetSelection(int sel)
{
/* old code start -------------------------------------------------------------------------------------------
    m_listbox->SetSelection(sel);
-- old code end / new code start ------------------------------------------------------------------------- */
// Modification by Christian Sturmlechner
    #if wxUSE_singlechoicedialog_ex
    if ( m_listbox )
    {
        m_listbox->SetSelection(sel);
    }
    else
    {
        m_radiobox->SetSelection(sel);
    }
    #else // !wxUSE_singlechoicedialog_ex
    m_listbox->SetSelection(sel);
    #endif // wxUSE_singlechoicedialog_ex / !wxUSE_singlechoicedialog_ex
// new code end ---------------------------------------------------------------------------------------------
    m_selection = sel;
}

void wxSingleChoiceDialog::OnOK(wxCommandEvent& WXUNUSED(event))
{
    DoChoice();
}

#ifndef __SMARTPHONE__
void wxSingleChoiceDialog::OnListBoxDClick(wxCommandEvent& WXUNUSED(event))
{
    DoChoice();
}
#endif

#ifdef __WXWINCE__
void wxSingleChoiceDialog::OnJoystickButtonDown(wxJoystickEvent& WXUNUSED(event))
{
    DoChoice();
}
#endif

void wxSingleChoiceDialog::DoChoice()
{
/* old code start -------------------------------------------------------------------------------------------
    m_selection = m_listbox->GetSelection();
    m_stringSelection = m_listbox->GetStringSelection();

    if ( m_listbox->HasClientUntypedData() )
        SetClientData(m_listbox->GetClientData(m_selection));
-- old code end / new code start ------------------------------------------------------------------------- */
// Modification by Christian Sturmlechner
    #if wxUSE_singlechoicedialog_ex
    if ( m_listbox )
    {
        m_selection = m_listbox->GetSelection();
        m_stringSelection = m_listbox->GetStringSelection();

        if ( m_listbox->HasClientUntypedData() )
        {
            SetClientData(m_listbox->GetClientData(m_selection));
        }
    }
    else
    {
        m_selection = m_radiobox->GetSelection();
        m_stringSelection = m_radiobox->GetString(m_selection);
    }
    #else // !wxUSE_singlechoicedialog_ex
    m_selection = m_listbox->GetSelection();
    m_stringSelection = m_listbox->GetStringSelection();

    if ( m_listbox->HasClientUntypedData() )
        SetClientData(m_listbox->GetClientData(m_selection));
    #endif // wxUSE_singlechoicedialog_ex / !wxUSE_singlechoicedialog_ex
// new code end ---------------------------------------------------------------------------------------------

    EndModal(wxID_OK);
}

// ----------------------------------------------------------------------------
// wxMultiChoiceDialog
// ----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxMultiChoiceDialog, wxDialog)

bool wxMultiChoiceDialog::Create( wxWindow *parent,
                                  const wxString& message,
                                  const wxString& caption,
                                  int n,
                                  const wxString *choices,
                                  long style,
                                  const wxPoint& pos )
{
    long styleLbox;
#if wxUSE_CHECKLISTBOX
    styleLbox = wxLB_ALWAYS_SB;
#else
    styleLbox = wxLB_ALWAYS_SB | wxLB_EXTENDED;
#endif

    if ( !wxAnyChoiceDialog::Create(parent, message, caption,
                                    n, choices,
                                    style, pos,
                                    styleLbox) )
        return false;

    return true;
}

bool wxMultiChoiceDialog::Create( wxWindow *parent,
                                  const wxString& message,
                                  const wxString& caption,
                                  const wxArrayString& choices,
                                  long style,
                                  const wxPoint& pos )
{
    wxCArrayString chs(choices);
    return Create( parent, message, caption, chs.GetCount(),
                   chs.GetStrings(), style, pos );
}

void wxMultiChoiceDialog::SetSelections(const wxArrayInt& selections)
{
// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
    wxASSERT( m_listbox );
// new code end ---------------------------------------------------------------------------------------------
#if wxUSE_CHECKLISTBOX
    wxCheckListBox* checkListBox = wxDynamicCast(m_listbox, wxCheckListBox);
    if (checkListBox)
    {
        // first clear all currently selected items
        size_t n,
            count = checkListBox->GetCount();
        for ( n = 0; n < count; ++n )
        {
            if (checkListBox->IsChecked(n))
                checkListBox->Check(n, false);
        }

        // now select the ones which should be selected
        count = selections.GetCount();
        for ( n = 0; n < count; n++ )
        {
            checkListBox->Check(selections[n]);
        }

        return;
    }
#endif

    // first clear all currently selected items
    size_t n,
           count = m_listbox->GetCount();
    for ( n = 0; n < count; ++n )
    {
        m_listbox->Deselect(n);
    }

    // now select the ones which should be selected
    count = selections.GetCount();
    for ( n = 0; n < count; n++ )
    {
        m_listbox->Select(selections[n]);
    }
}

bool wxMultiChoiceDialog::TransferDataFromWindow()
{
    m_selections.Empty();

#if wxUSE_CHECKLISTBOX
    wxCheckListBox* checkListBox = wxDynamicCast(m_listbox, wxCheckListBox);
    if (checkListBox)
    {
        size_t count = checkListBox->GetCount();
        for ( size_t n = 0; n < count; n++ )
        {
            if ( checkListBox->IsChecked(n) )
                m_selections.Add(n);
        }
        return true;
    }
#endif

    size_t count = m_listbox->GetCount();
    for ( size_t n = 0; n < count; n++ )
    {
        if ( m_listbox->IsSelected(n) )
            m_selections.Add(n);
    }

    return true;
}

#if wxUSE_CHECKLISTBOX

wxListBoxBase *wxMultiChoiceDialog::CreateList(int n, const wxString *choices, long styleLbox)
{
	wxSize size = wxDefaultSize;
	if (wxSystemSettings::GetScreenType() > wxSYS_SCREEN_PDA)
		size = wxSize(300, 200);

    return new wxCheckListBox( this, wxID_LISTBOX,
                               wxDefaultPosition, size,
                               n, choices,
                               styleLbox );
}

#endif // wxUSE_CHECKLISTBOX

#endif // wxUSE_CHOICEDLG
