/////////////////////////////////////////////////////////////////////////////
// Name:        stattext.cpp
// Purpose:
// Author:      Robert Roebling
// Modified by: Christian Sturmlechner 2010-10-02 - 2018-01-18
// Id:          $Id: stattext.cpp 52973 2008-04-02 09:36:53Z VS $
// Copyright:   (c) 1998 Robert Roebling
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#if wxUSE_STATTEXT

#include "wx/stattext.h"
#include "wx/gtk/private.h"

#include "gdk/gdk.h"
#include "gtk/gtk.h"
// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#if wxUSE_HACKFORSTATTEXTPOSITION
  #include "wx/apptrait.h"
  #include "wx/app.h"
#endif // wxUSE_HACKFORSTATTEXTPOSITION
// new code end ---------------------------------------------------------------------------------------------

extern "C"
void wxgtk_window_size_request_callback(GtkWidget *widget,
                                        GtkRequisition *requisition,
                                        wxWindow *win);

//-----------------------------------------------------------------------------
// wxStaticText
//-----------------------------------------------------------------------------

IMPLEMENT_DYNAMIC_CLASS(wxStaticText,wxControl)

wxStaticText::wxStaticText()
{
}

wxStaticText::wxStaticText(wxWindow *parent,
                           wxWindowID id,
                           const wxString &label,
                           const wxPoint &pos,
                           const wxSize &size,
                           long style,
                           const wxString &name)
{
  Create( parent, id, label, pos, size, style, name );
}

bool wxStaticText::Create(wxWindow *parent,
                          wxWindowID id,
                          const wxString &label,
                          const wxPoint &pos,
                          const wxSize &size,
                          long style,
                          const wxString &name )
{
    m_needParent = TRUE;

    if (!PreCreation( parent, pos, size ) ||
        !CreateBase( parent, id, pos, size, style, wxDefaultValidator, name ))
    {
        wxFAIL_MSG( wxT("wxStaticText creation failed") );
        return FALSE;
    }

/* old code start -------------------------------------------------------------------------------------------
    const wxString labelGTK = GTKConvertMnemonics(label);
-- old code end / new code start ------------------------------------------------------------------------- */
// Modification by Christian Sturmlechner
    wxString labelGTK;
    if ( m_windowStyle & wxST_KEEP_MNEMONICS )
    {
      labelGTK = label;
    }
    else
    {
      labelGTK = GTKConvertMnemonics(label);
    }
// new code end ---------------------------------------------------------------------------------------------
    m_label = label;
    m_widget = gtk_label_new_with_mnemonic(wxGTK_CONV(labelGTK));

    GtkJustification justify;
    if ( style & wxALIGN_CENTER )
      justify = GTK_JUSTIFY_CENTER;
    else if ( style & wxALIGN_RIGHT )
      justify = GTK_JUSTIFY_RIGHT;
    else // wxALIGN_LEFT is 0
      justify = GTK_JUSTIFY_LEFT;

    if (GetLayoutDirection() == wxLayout_RightToLeft)
    {
         if (justify == GTK_JUSTIFY_RIGHT)
            justify = GTK_JUSTIFY_LEFT;
         if (justify == GTK_JUSTIFY_LEFT)
            justify = GTK_JUSTIFY_RIGHT;
    }

    gtk_label_set_justify(GTK_LABEL(m_widget), justify);

    // GTK_JUSTIFY_LEFT is 0, RIGHT 1 and CENTER 2
    static const float labelAlignments[] = { 0.0, 1.0, 0.5 };
    gtk_misc_set_alignment(GTK_MISC(m_widget), labelAlignments[justify], 0.0);

    gtk_label_set_line_wrap( GTK_LABEL(m_widget), TRUE );

    m_parent->DoAddChild( this );

    PostCreation(size);

    // the bug below only happens with GTK 2
    if ( justify != GTK_JUSTIFY_LEFT )
    {
        // if we let GTK call wxgtk_window_size_request_callback the label
        // always shrinks to its minimal size for some reason and so no
        // alignment except the default left doesn't work (in fact it does,
        // but you don't see it)
        g_signal_handlers_disconnect_by_func (m_widget,
                                              (gpointer) wxgtk_window_size_request_callback,
                                              this);
    }

// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#if wxUSE_HACKFORSTATTEXTPOSITION
    // This hack is only used for some gtk-versions and leads to errors with the others.
    // Because of this errors it is not used in doubt.
    // Results of my tests: gtk+-2.12.9 (OpenSuse-11.0): no need
    //                           2.12.1 (FreeBSD-7.0) no need
    bUseHackForStattextPosition = false;
    int iToolkitVerMaj, iToolkitVerMin;
    if ( wxTheApp != NULL )
    {
       wxTheApp->GetTraits()->GetToolkitVersion( &iToolkitVerMaj, &iToolkitVerMin );
    }
    if ( (    iToolkitVerMaj == 2
           && iToolkitVerMin >= 13
         )
         // || iToolkitVerMaj >= 3 - needs testing
       )
    {
      if (    HasFlag(wxST_NO_AUTORESIZE)
           && (    HasFlag(wxALIGN_CENTRE)
                || HasFlag(wxALIGN_RIGHT)
              )
         )
      {
        bUseHackForStattextPosition = true;
        PosOrig = pos;
        SizeOrig = GetSize();
        SetLabel(label);
      }
    }
#endif // wxUSE_HACKFORSTATTEXTPOSITION
// new code end ---------------------------------------------------------------------------------------------
    return TRUE;
}

wxString wxStaticText::GetLabel() const
{
    GtkLabel *label = GTK_LABEL(m_widget);
    wxString str = wxGTK_CONV_BACK( gtk_label_get_text( label ) );

    return wxString(str);
}

void wxStaticText::SetLabel( const wxString &label )
{
    wxCHECK_RET( m_widget != NULL, wxT("invalid static text") );

/* old code start -------------------------------------------------------------------------------------------
    GTKSetLabelForLabel(GTK_LABEL(m_widget), label);
-- old code end / new code start ------------------------------------------------------------------------- */
// Modification by Christian Sturmlechner
    if ( m_windowStyle & wxST_KEEP_MNEMONICS )
    {
        // almost a copy of wxControl::GTKSetLabelForLabel(GtkLabel *w, const wxString& label) in src/gtk/control.cpp
        wxControl::SetLabel(label);
        wxString labelGTK (label );
        gtk_label_set_text_with_mnemonic(GTK_LABEL(m_widget), wxGTK_CONV(labelGTK));
    }
    else
    {
        GTKSetLabelForLabel(GTK_LABEL(m_widget), label);
    }
// new code end ---------------------------------------------------------------------------------------------

    // adjust the label size to the new label unless disabled
    if ( !HasFlag(wxST_NO_AUTORESIZE) )
    {
        InvalidateBestSize();
        SetSize( GetBestSize() );
    }
// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
// This is not a full implementation of the workaround (see mysetup.h), for example it lacks the
// possibility to change the alignment after the first creation of the control.
#if wxUSE_HACKFORSTATTEXTPOSITION
    if ( bUseHackForStattextPosition )
    {
      wxSize bestsize = GetBestSize();
      SetSize( bestsize );

      wxPoint pointNewPos;
      pointNewPos.y = PosOrig.y;
      if ( HasFlag(wxALIGN_CENTRE) )
      {
        pointNewPos.x = PosOrig.x + ( (SizeOrig.GetWidth()-bestsize.GetWidth())/2 );
      }
      else
      {
        pointNewPos.x = PosOrig.x + ( (SizeOrig.GetWidth()-bestsize.GetWidth()) );
      }
      Move( pointNewPos );
    }
#endif // wxUSE_HACKFORSTATTEXTPOSITION
// new code end ---------------------------------------------------------------------------------------------
}

// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#if wxUSE_HACKFORSTATTEXTPOSITION
void wxStaticText::SetPositionEx(const wxPoint& pt)
{
    if ( bUseHackForStattextPosition )
    {
        PosOrig = pt;
        wxSize bestsize = GetBestSize();
        SetSize( bestsize );

        wxPoint pointNewPos;
        pointNewPos.y = PosOrig.y;
        if ( HasFlag(wxALIGN_CENTRE) )
        {
          pointNewPos.x = PosOrig.x + ( (SizeOrig.GetWidth()-bestsize.GetWidth())/2 );
        }
        else
        {
          pointNewPos.x = PosOrig.x + ( (SizeOrig.GetWidth()-bestsize.GetWidth()) );
        }
        Move( pointNewPos );
    }
    else
    {
        SetPosition(pt);
    }
}

void wxStaticText::SetSizeEx( int width, int height )
{
    if ( bUseHackForStattextPosition )
    {
        SizeOrig = wxSize(width,height);
        SetPositionEx( PosOrig );
    }
    else
    {
        SetSize(width,height);
    }
}
#endif // wxUSE_HACKFORSTATTEXTPOSITION
// new code end ---------------------------------------------------------------------------------------------

bool wxStaticText::SetFont( const wxFont &font )
{
    bool ret = wxControl::SetFont(font);

    // adjust the label size to the new label unless disabled
    if (!HasFlag(wxST_NO_AUTORESIZE))
    {
        InvalidateBestSize();
        SetSize( GetBestSize() );
    }
    return ret;
}

void wxStaticText::DoSetSize(int x, int y,
                           int width, int height,
                           int sizeFlags )
{
    wxControl::DoSetSize( x, y, width, height, sizeFlags );
}

wxSize wxStaticText::DoGetBestSize() const
{
    // Do not return any arbitrary default value...
    wxASSERT_MSG( m_widget, wxT("wxStaticText::DoGetBestSize called before creation") );

    // GetBestSize is supposed to return unwrapped size but calling
    // gtk_label_set_line_wrap() from here is a bad idea as it queues another
    // size request by calling gtk_widget_queue_resize() and we end up in
    // infinite loop sometimes (notably when the control is in a toolbar)
    GTK_LABEL(m_widget)->wrap = FALSE;

    GtkRequisition req;
    req.width = -1;
    req.height = -1;
    (* GTK_WIDGET_CLASS( GTK_OBJECT_GET_CLASS(m_widget) )->size_request )
        (m_widget, &req );

    GTK_LABEL(m_widget)->wrap = TRUE; // restore old value

    // Adding 1 to width to workaround GTK sometimes wrapping the text needlessly
    return wxSize (req.width+1, req.height);
}

bool wxStaticText::SetForegroundColour(const wxColour& colour)
{
    // First, we call the base class member
    wxControl::SetForegroundColour(colour);
    // Then, to force the color change, we set the label with the current label
    SetLabel(GetLabel());
    return true;
}

bool wxStaticText::GTKWidgetNeedsMnemonic() const
{
    return true;
}

void wxStaticText::GTKWidgetDoSetMnemonic(GtkWidget* w)
{
    gtk_label_set_mnemonic_widget(GTK_LABEL(m_widget), w);
}

// static
wxVisualAttributes
wxStaticText::GetClassDefaultAttributes(wxWindowVariant WXUNUSED(variant))
{
    return GetDefaultAttributesFromGTKWidget(gtk_label_new);
}

#endif // wxUSE_STATTEXT
