/////////////////////////////////////////////////////////////////////////////
// Name:        helpctrl.cpp
// Purpose:     wxHtmlHelpController
// Notes:       Based on htmlhelp.cpp, implementing a monolithic
//              HTML Help controller class,  by Vaclav Slavik
// Author:      Harm van der Heijden and Vaclav Slavik
// Modified by: Christian Sturmlechner 2011-02-27 - 2017-04-02
// RCS-ID:      $Id: helpctrl.cpp 52470 2008-03-13 23:29:27Z VS $
// Copyright:   (c) Harm van der Heijden and Vaclav Slavik
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#if wxUSE_WXHTML_HELP

#ifndef WX_PRECOMP
    #include "wx/app.h"
    #include "wx/intl.h"
#endif // WX_PRECOMP

#include "wx/busyinfo.h"
#include "wx/html/helpctrl.h"
#include "wx/html/helpwnd.h"
#if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------------
#include "wx/html/helpfrm.h"
#include "wx/html/helpdlg.h"
#endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------

#if wxUSE_HELP
    #include "wx/tipwin.h"
#endif

#if wxUSE_LIBMSPACK
#include "wx/html/forcelnk.h"
FORCE_LINK(wxhtml_chm_support)
#endif

// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#if !defined( WXUNUSED_IF_NOT_WXGUI )
  #if !wxUSE_GUI // console o haiku
    #if !defined( __MYCONSOLE__ ) && !defined( __MYBEOS__ )
      #error wrong definition of WXUNUSED_IF_NOT_WXGUI
    #endif
    #define WXUNUSED_IF_NOT_WXGUI( parameter )
  #else // wxUSE_GUI
    #define WXUNUSED_IF_NOT_WXGUI( parameter ) parameter
  #endif // !wxUSE_GUI / wxUSE_GUI
#endif // !WXUNUSED_IF_NOT_WXGUI
// new code end ---------------------------------------------------------------------------------------------

#if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------------
IMPLEMENT_DYNAMIC_CLASS(wxHtmlHelpController, wxHelpControllerBase)
#endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------

/* old code start -------------------------------------------------------------------------------------------
wxHtmlHelpController::wxHtmlHelpController(int style, wxWindow* parentWindow):
    wxHelpControllerBase(parentWindow)
-- old code end / new code start (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------ */
#if wxUSE_GUI
wxHtmlHelpController::wxHtmlHelpController(int style, wxWindow* parentWindow):
    wxHelpControllerBase(parentWindow)
#else // !wxUSE_GUI
wxHtmlHelpController::wxHtmlHelpController(void) :
    wxHelpControllerBase(NULL)
#endif // wxUSE_GUI / !wxUSE_GUI
// new code end ---------------------------------------------------------------------------------------------
{
    m_helpWindow = NULL;
    #if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------
    m_helpFrame = NULL;
    m_helpDialog = NULL;
    #endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------
    m_Config = NULL;
    m_ConfigRoot = wxEmptyString;
    m_titleFormat = _("Help: %s");
    #if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------
    m_FrameStyle = style;
    #endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------
}

wxHtmlHelpController::~wxHtmlHelpController()
{
// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#if wxUSE_CONFIG
// new code end ---------------------------------------------------------------------------------------------
    if (m_Config)
        WriteCustomization(m_Config, m_ConfigRoot);
// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#endif // wxUSE_CONFIG
// new code end ---------------------------------------------------------------------------------------------
    if (m_helpWindow)
        DestroyHelpWindow();
}


#if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------------
void wxHtmlHelpController::DestroyHelpWindow()
{
    if (m_FrameStyle & wxHF_EMBEDDED)
        return;

    // Find top-most parent window
    // If a modal dialog
    wxWindow* parent = FindTopLevelWindow();
    if (parent)
    {
        wxDialog* dialog = wxDynamicCast(parent, wxDialog);
        if (dialog && dialog->IsModal())
        {
            dialog->EndModal(wxID_OK);
        }
        parent->Destroy();
        m_helpWindow = NULL;
    }
    m_helpDialog = NULL;
    m_helpFrame = NULL;
}
#endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------

#if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------------
void wxHtmlHelpController::OnCloseFrame(wxCloseEvent& evt)
{
// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#if wxUSE_CONFIG
// new code end ---------------------------------------------------------------------------------------------
    if (m_Config)
        WriteCustomization(m_Config, m_ConfigRoot);
// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#endif // wxUSE_CONFIG
// new code end ---------------------------------------------------------------------------------------------

    evt.Skip();

    OnQuit();

    if ( m_helpWindow )
        m_helpWindow->SetController(NULL);
    m_helpWindow = NULL;
    m_helpDialog = NULL;
    m_helpFrame = NULL;
}
#endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------

void wxHtmlHelpController::SetTitleFormat(const wxString& title)
{
    m_titleFormat = title;
    #if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------
    wxHtmlHelpFrame* frame = wxDynamicCast(FindTopLevelWindow(), wxHtmlHelpFrame);
    wxHtmlHelpDialog* dialog = wxDynamicCast(FindTopLevelWindow(), wxHtmlHelpDialog);
    if (frame)
    {
        frame->SetTitleFormat(title);
    }
    else if (dialog)
        dialog->SetTitleFormat(title);
    #endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------
}

#if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------------
// Find the top-most parent window
wxWindow* wxHtmlHelpController::FindTopLevelWindow()
{
    wxWindow* parent = m_helpWindow;
    while (parent && !parent->IsTopLevel())
    {
        parent = parent->GetParent();
    }
    return parent;
}
#endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------

bool wxHtmlHelpController::AddBook(const wxFileName& book_file, bool show_wait_msg)
{
    return AddBook(wxFileSystem::FileNameToURL(book_file), show_wait_msg);
}

bool wxHtmlHelpController::AddBook(const wxString& book, bool show_wait_msg)
{
    #if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------
    wxBusyCursor cur;
    #endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------
#if wxUSE_BUSYINFO
    wxBusyInfo* busy = NULL;
    wxString info;
    if (show_wait_msg)
    {
        info.Printf(_("Adding book %s"), book.c_str());
        busy = new wxBusyInfo(info);
    }
#endif
    bool retval = m_helpData.AddBook(book);
#if wxUSE_BUSYINFO
    if (show_wait_msg)
        delete busy;
#else
    wxUnusedVar(show_wait_msg);
#endif
    if (m_helpWindow)
        m_helpWindow->RefreshLists();
    return retval;
}

#if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------------
wxHtmlHelpFrame* wxHtmlHelpController::CreateHelpFrame(wxHtmlHelpData *data)
{
    wxHtmlHelpFrame* frame = new wxHtmlHelpFrame(data);
    frame->SetController(this);
    frame->Create(m_parentWindow, -1, wxEmptyString, m_FrameStyle, m_Config, m_ConfigRoot);
    frame->SetTitleFormat(m_titleFormat);
    m_helpFrame = frame;
    return frame;
}

wxHtmlHelpDialog* wxHtmlHelpController::CreateHelpDialog(wxHtmlHelpData *data)
{
    wxHtmlHelpDialog* dialog = new wxHtmlHelpDialog(data);
    dialog->SetController(this);
    dialog->SetTitleFormat(m_titleFormat);
    dialog->Create(m_parentWindow, -1, wxEmptyString, m_FrameStyle);
    m_helpDialog = dialog;
    return dialog;
}

wxWindow* wxHtmlHelpController::CreateHelpWindow()
{
    if (m_helpWindow)
    {
        if (m_FrameStyle & wxHF_EMBEDDED)
            return m_helpWindow;

        wxWindow* topLevelWindow = FindTopLevelWindow();
        if (topLevelWindow)
            topLevelWindow->Raise();
        return m_helpWindow;
    }

// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#if wxUSE_CONFIG
// new code end ---------------------------------------------------------------------------------------------
    if (m_Config == NULL)
    {
        m_Config = wxConfigBase::Get(false);
        if (m_Config != NULL)
            m_ConfigRoot = _T("wxWindows/wxHtmlHelpController");
    }
// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#endif // wxUSE_CONFIG
// new code end ---------------------------------------------------------------------------------------------

    if (m_FrameStyle & wxHF_DIALOG)
    {
        wxHtmlHelpDialog* dialog = CreateHelpDialog(&m_helpData);
        m_helpWindow = dialog->GetHelpWindow();
    }
    else if ((m_FrameStyle & wxHF_EMBEDDED) && m_parentWindow)
    {
        m_helpWindow = new wxHtmlHelpWindow(m_parentWindow, -1, wxDefaultPosition, wxDefaultSize,
            wxTAB_TRAVERSAL|wxNO_BORDER, m_FrameStyle, &m_helpData);
    }
    else // wxHF_FRAME
    {
        wxHtmlHelpFrame* frame = CreateHelpFrame(&m_helpData);
        m_helpWindow = frame->GetHelpWindow();
        frame->Show(true);
    }

    return m_helpWindow;
}
#endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------

// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#if wxUSE_CONFIG
// new code end ---------------------------------------------------------------------------------------------
#if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------------
void wxHtmlHelpController::ReadCustomization(wxConfigBase* cfg, const wxString& path)
#else // !wxUSE_GUI new code start (Modification by Christian Sturmlechner - STURMLECHNER_BASE) -------------
void wxHtmlHelpController::ReadCustomization(wxConfigBase* WXUNUSED(cfg), const wxString& WXUNUSED(path))
#endif // wxUSE_GUI / !wxUSE_GUI // new code end ------------------------------------------------------------
{
    #if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------
    /* should not be called by the user; call UseConfig, and the controller
     * will do the rest */
    if (m_helpWindow && cfg)
        m_helpWindow->ReadCustomization(cfg, path);
    #endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------
}

#if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------------
void wxHtmlHelpController::WriteCustomization(wxConfigBase* cfg, const wxString& path)
#else // !wxUSE_GUI new code start (Modification by Christian Sturmlechner - STURMLECHNER_BASE) -------------
void wxHtmlHelpController::WriteCustomization(wxConfigBase* WXUNUSED(cfg), const wxString& WXUNUSED(path))
#endif // wxUSE_GUI / !wxUSE_GUI // new code end ------------------------------------------------------------
{
    #if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------
    /* typically called by the controllers OnCloseFrame handler */
    if (m_helpWindow && cfg)
        m_helpWindow->WriteCustomization(cfg, path);
    #endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------
}
// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#endif // wxUSE_CONFIG
// new code end ---------------------------------------------------------------------------------------------

void wxHtmlHelpController::UseConfig(wxConfigBase *config, const wxString& rootpath)
{
    m_Config = config;
    m_ConfigRoot = rootpath;
// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#if wxUSE_CONFIG
// new code end ---------------------------------------------------------------------------------------------
    #if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------
    if (m_helpWindow) m_helpWindow->UseConfig(config, rootpath);
    #endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------
    ReadCustomization(config, rootpath);
// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
#endif // wxUSE_CONFIG
// new code end ---------------------------------------------------------------------------------------------
}

//// Backward compatibility with wxHelpController API

bool wxHtmlHelpController::Initialize(const wxString& file)
{
    wxString dir, filename, ext;
    wxSplitPath(file, & dir, & filename, & ext);

    if (!dir.empty())
        dir = dir + wxFILE_SEP_PATH;

    // Try to find a suitable file
    wxString actualFilename = dir + filename + wxString(wxT(".zip"));
    if (!wxFileExists(actualFilename))
    {
        actualFilename = dir + filename + wxString(wxT(".htb"));
        if (!wxFileExists(actualFilename))
        {
            actualFilename = dir + filename + wxString(wxT(".hhp"));
            if (!wxFileExists(actualFilename))
            {
#if wxUSE_LIBMSPACK
                actualFilename = dir + filename + wxString(wxT(".chm"));
                if (!wxFileExists(actualFilename))
#endif
                    return false;
            }
        }
    }
    return AddBook(wxFileName(actualFilename));
}

bool wxHtmlHelpController::LoadFile(const wxString& WXUNUSED(file))
{
    // Don't reload the file or we'll have it appear again, presumably.
    return true;
}

bool wxHtmlHelpController::DisplaySection(int WXUNUSED_IF_NOT_WXGUI(sectionNo)) // WXUNUSED_IF_NOT_WXGUI <- Sturmlechner
{
/* old code start -------------------------------------------------------------------------------------------
    return Display(sectionNo);
-- old code end / new code start (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------ */
    #if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------
    return Display(sectionNo);
    #else // !wxUSE_GUI
    return false;
    #endif // wxUSE_GUI / !wxUSE_GUI
// new code end ---------------------------------------------------------------------------------------------
}

bool wxHtmlHelpController::DisplayTextPopup(const wxString& text, const wxPoint& WXUNUSED(pos))
{
#if wxUSE_TIPWINDOW
    static wxTipWindow* s_tipWindow = NULL;

    if (s_tipWindow)
    {
        // Prevent s_tipWindow being nulled in OnIdle,
        // thereby removing the chance for the window to be closed by ShowHelp
        s_tipWindow->SetTipWindowPtr(NULL);
        s_tipWindow->Close();
    }
    s_tipWindow = NULL;

    if ( !text.empty() )
    {
        s_tipWindow = new wxTipWindow(wxTheApp->GetTopWindow(), text, 100, & s_tipWindow);

        return true;
    }
#else
    wxUnusedVar(text);
#endif // wxUSE_TIPWINDOW

    return false;
}

#if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------------
void wxHtmlHelpController::SetHelpWindow(wxHtmlHelpWindow* helpWindow)
{
    m_helpWindow = helpWindow;
    if (helpWindow)
        helpWindow->SetController(this);
}
#endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------

void wxHtmlHelpController::SetFrameParameters(const wxString& title,
                                   const wxSize& WXUNUSED_IF_NOT_WXGUI(size), // WXUNUSED_IF_NOT_WXGUI <- Sturmlechner
                                   const wxPoint& WXUNUSED_IF_NOT_WXGUI(pos), // WXUNUSED_IF_NOT_WXGUI <- Sturmlechner
                                   bool WXUNUSED(newFrameEachTime))
{
    SetTitleFormat(title);
    #if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------
    wxHtmlHelpFrame* frame = wxDynamicCast(FindTopLevelWindow(), wxHtmlHelpFrame);
    wxHtmlHelpDialog* dialog = wxDynamicCast(FindTopLevelWindow(), wxHtmlHelpDialog);
    if (frame)
        frame->SetSize(pos.x, pos.y, size.x, size.y);
    else if (dialog)
        dialog->SetSize(pos.x, pos.y, size.x, size.y);
    #endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------
}

wxFrame* wxHtmlHelpController::GetFrameParameters(wxSize *WXUNUSED_IF_NOT_WXGUI(size), // WXUNUSED_IF_NOT_WXGUI <- Sturmlechner
                                   wxPoint *WXUNUSED_IF_NOT_WXGUI(pos), // WXUNUSED_IF_NOT_WXGUI <- Sturmlechner
                                   bool *newFrameEachTime)
{
    if (newFrameEachTime)
        (* newFrameEachTime) = false;

    #if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------
    wxHtmlHelpFrame* frame = wxDynamicCast(FindTopLevelWindow(), wxHtmlHelpFrame);
    wxHtmlHelpDialog* dialog = wxDynamicCast(FindTopLevelWindow(), wxHtmlHelpDialog);
    if (frame)
    {
        if (size)
            (* size) = frame->GetSize();
        if (pos)
            (* pos) = frame->GetPosition();
        return frame;
    }
    else if (dialog)
    {
        if (size)
            (* size) = dialog->GetSize();
        if (pos)
            (* pos) = dialog->GetPosition();
        return NULL;
    }
    #endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------
    return NULL;
}

#if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------------
bool wxHtmlHelpController::Quit()
{
    DestroyHelpWindow();
    return true;
}
#endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------

// Make the help controller's frame 'modal' if
// needed
void wxHtmlHelpController::MakeModalIfNeeded()
{
#if wxUSE_GUI // *** Sturmlechner ausdefinen todo
    if ((m_FrameStyle & wxHF_EMBEDDED) == 0)
    {
        wxHtmlHelpFrame* frame = wxDynamicCast(FindTopLevelWindow(), wxHtmlHelpFrame);
        wxHtmlHelpDialog* dialog = wxDynamicCast(FindTopLevelWindow(), wxHtmlHelpDialog);
        if (frame)
            frame->AddGrabIfNeeded();
        else if (dialog && (m_FrameStyle & wxHF_MODAL))
        {
            dialog->ShowModal();
        }
    }
#endif // ***
}

bool wxHtmlHelpController::Display(const wxString& x)
{
    #if 1 // *** wxUSE_GUI || defined( __MYBEOS__ ) // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------
    CreateHelpWindow();
    bool success = m_helpWindow->Display(x);
    MakeModalIfNeeded();
    return success;
// new code start (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------------------
    #else // !wxUSE_GUI
    return false;
    #endif // wxUSE_GUI / wxUSE_GUI
// new code end ---------------------------------------------------------------------------------------------
}

bool wxHtmlHelpController::Display(int id)
{
    #if 1 // *** wxUSE_GUI || defined( __MYBEOS__ ) // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------
    CreateHelpWindow();
    bool success = m_helpWindow->Display(id);
    MakeModalIfNeeded();
    return success;
// new code start (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------------------
    #else // !wxUSE_GUI
    return false;
    #endif // wxUSE_GUI / wxUSE_GUI
// new code end ---------------------------------------------------------------------------------------------
}

bool wxHtmlHelpController::DisplayContents()
{
    #if 1 // *** wxUSE_GUI || defined( __MYBEOS__ ) // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------
    CreateHelpWindow();
    bool success = m_helpWindow->DisplayContents();
    MakeModalIfNeeded();
    return success;
// new code start (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------------------
    #else // !wxUSE_GUI
    return false;
    #endif // wxUSE_GUI / wxUSE_GUI
// new code end ---------------------------------------------------------------------------------------------
}

bool wxHtmlHelpController::DisplayIndex()
{
    #if 1 // *** wxUSE_GUI || defined( __MYBEOS__ ) // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------
    CreateHelpWindow();
    bool success = m_helpWindow->DisplayIndex();
    MakeModalIfNeeded();
    return success;
// new code start (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ------------------------------
    #else // !wxUSE_GUI
    return false;
    #endif // wxUSE_GUI / wxUSE_GUI
// new code end ---------------------------------------------------------------------------------------------
}

bool wxHtmlHelpController::KeywordSearch(const wxString& keyword,
                                         wxHelpSearchMode mode)
{
    CreateHelpWindow();
    bool success = m_helpWindow->KeywordSearch(keyword, mode);
    MakeModalIfNeeded();
    return success;
}

#if wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------------
/*
 * wxHtmlModalHelp
 * A convenience class, to use like this:
 *
 * wxHtmlModalHelp help(parent, helpFile, topic);
 */

wxHtmlModalHelp::wxHtmlModalHelp(wxWindow* parent, const wxString& helpFile, const wxString& topic, int style)
{
    // Force some mandatory styles
    style |= wxHF_DIALOG | wxHF_MODAL;

    wxHtmlHelpController controller(style, parent);
    controller.Initialize(helpFile);

    if (topic.IsEmpty())
        controller.DisplayContents();
    else
        controller.DisplaySection(topic);
}
#endif // wxUSE_GUI // new code (Modification by Christian Sturmlechner - STURMLECHNER_BASE) ----------------

#endif // wxUSE_WXHTML_HELP

