/////////////////////////////////////////////////////////////////////////////
// Name:        src/msw/main.cpp
// Purpose:     WinMain/DllMain
// Author:      Julian Smart
// Modified by: Christian Sturmlechner 2007-01-28 - 2017-05-04
// Created:     04/01/98
// RCS-ID:      $Id: main.cpp 44727 2007-03-10 17:24:09Z VZ $
// Copyright:   (c) Julian Smart
// Licence:     wxWindows licence
/////////////////////////////////////////////////////////////////////////////

// ============================================================================
// declarations
// ============================================================================

// ----------------------------------------------------------------------------
// headers
// ----------------------------------------------------------------------------

// For compilers that support precompilation, includes "wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/event.h"
    #include "wx/app.h"
    #include "wx/utils.h"
#endif //WX_PRECOMP

#include "wx/cmdline.h"
#include "wx/scopeguard.h"

#include "wx/msw/private.h"
#include "wx/msw/seh.h"

#if wxUSE_ON_FATAL_EXCEPTION
    #include "wx/datetime.h"
    #include "wx/msw/crashrpt.h"
#endif // wxUSE_ON_FATAL_EXCEPTION

#ifdef __WXWINCE__
    // there is no ExitProcess() under CE but exiting the main thread has the
    // same effect
    #ifndef ExitProcess
        #define ExitProcess ExitThread
    #endif
#endif // __WXWINCE__

#ifdef __BORLANDC__
    // BC++ has to be special: its run-time expects the DLL entry point to be
    // named DllEntryPoint instead of the (more) standard DllMain
    #define DllMain DllEntryPoint
#endif // __BORLANDC__

#if defined(__WXMICROWIN__)
    #define HINSTANCE HANDLE
#endif

// defined in common/init.cpp
extern int wxEntryReal(int& argc, wxChar **argv);

// ============================================================================
// implementation: various entry points
// ============================================================================

#if wxUSE_BASE

#if wxUSE_ON_FATAL_EXCEPTION && defined(__VISUALC__) && !defined(__WXWINCE__)
    // VC++ (at least from 4.0 up to version 7.1) is incredibly broken in that
    // a "catch ( ... )" will *always* catch SEH exceptions in it even though
    // it should have never been the case... to prevent such catches from
    // stealing the exceptions from our wxGlobalSEHandler which is only called
    // if the exception is not handled elsewhere, we have to also call it from
    // a special SEH translator function which is called by VC CRT when a Win32
    // exception occurs

    // this warns that /EHa (async exceptions) should be used when using
    // _set_se_translator but, in fact, this doesn't seem to change anything
    // with VC++ up to 8.0
    #if _MSC_VER <= 1400
        #pragma warning(disable:4535)
    #endif

    // note that the SE translator must be called wxSETranslator!
    #define DisableAutomaticSETranslator() _set_se_translator(wxSETranslator)
#else // !__VISUALC__
    #define DisableAutomaticSETranslator()
#endif // __VISUALC__/!__VISUALC__

// ----------------------------------------------------------------------------
// wrapper wxEntry catching all Win32 exceptions occurring in a wx program
// ----------------------------------------------------------------------------

// wrap real wxEntry in a try-except block to be able to call
// OnFatalException() if necessary
#if wxUSE_ON_FATAL_EXCEPTION

// global pointer to exception information, only valid inside OnFatalException,
// used by wxStackWalker and wxCrashReport
extern EXCEPTION_POINTERS *wxGlobalSEInformation = NULL;

// flag telling us whether the application wants to handle exceptions at all
static bool gs_handleExceptions = false;

static void wxFatalExit()
{
    // use the same exit code as abort()
    ::ExitProcess(3);
}

unsigned long wxGlobalSEHandler(EXCEPTION_POINTERS *pExcPtrs)
{
    if ( gs_handleExceptions && wxTheApp )
    {
        // store the pointer to exception info
        wxGlobalSEInformation = pExcPtrs;

        // give the user a chance to do something special about this
        wxSEH_TRY
        {
            wxTheApp->OnFatalException();
        }
        wxSEH_IGNORE      // ignore any exceptions inside the exception handler

        wxGlobalSEInformation = NULL;

        // this will execute our handler and terminate the process
        return EXCEPTION_EXECUTE_HANDLER;
    }

    return EXCEPTION_CONTINUE_SEARCH;
}

#ifdef __VISUALC__

void wxSETranslator(unsigned int WXUNUSED(code), EXCEPTION_POINTERS *ep)
{
    switch ( wxGlobalSEHandler(ep) )
    {
        default:
            wxFAIL_MSG( _T("unexpected wxGlobalSEHandler() return value") );
            // fall through

        case EXCEPTION_EXECUTE_HANDLER:
            // if wxApp::OnFatalException() had been called we should exit the
            // application -- but we shouldn't kill our host when we're a DLL
#ifndef WXMAKINGDLL
            wxFatalExit();
#endif // not a DLL
            break;

        case EXCEPTION_CONTINUE_SEARCH:
            // we're called for each "catch ( ... )" and if we (re)throw from
            // here, the catch handler body is not executed, so the effect is
            // as if had inhibited translation of SE to C++ ones because the
            // handler will never see any structured exceptions
            throw;
    }
}

#endif // __VISUALC__

bool wxHandleFatalExceptions(bool doit)
{
    // assume this can only be called from the main thread
    gs_handleExceptions = doit;

#if wxUSE_CRASHREPORT
    if ( doit )
    {
        // try to find a place where we can put out report file later
        wxChar fullname[MAX_PATH];
        if ( !::GetTempPath(WXSIZEOF(fullname), fullname) )
        {
            wxLogLastError(_T("GetTempPath"));

            // when all else fails...
            wxStrcpy(fullname, _T("c:\\"));
        }

        // use PID and date to make the report file name more unique
        wxString name = wxString::Format
                        (
                            _T("%s_%s_%lu.dmp"),
                            wxTheApp ? wxTheApp->GetAppName().c_str()
                                     : _T("wxwindows"),
                            wxDateTime::Now().Format(_T("%Y%m%dT%H%M%S")).c_str(),
                            ::GetCurrentProcessId()
                        );

        wxStrncat(fullname, name, WXSIZEOF(fullname) - wxStrlen(fullname) - 1);

        wxCrashReport::SetFileName(fullname);
    }
#endif // wxUSE_CRASHREPORT

    return true;
}

int wxEntry(int& argc, wxChar **argv)
{
    DisableAutomaticSETranslator();

    wxSEH_TRY
    {
        return wxEntryReal(argc, argv);
    }
    wxSEH_HANDLE(-1)
}

#else // !wxUSE_ON_FATAL_EXCEPTION

#if defined(__VISUALC__) && !defined(__WXWINCE__)

static void
wxSETranslator(unsigned int WXUNUSED(code), EXCEPTION_POINTERS * WXUNUSED(ep))
{
    // see wxSETranslator() version for wxUSE_ON_FATAL_EXCEPTION above
    throw;
}

#endif // __VISUALC__

int wxEntry(int& argc, wxChar **argv)
{
    DisableAutomaticSETranslator();

    return wxEntryReal(argc, argv);
}

#endif // wxUSE_ON_FATAL_EXCEPTION/!wxUSE_ON_FATAL_EXCEPTION

#endif // wxUSE_BASE

#if wxUSE_GUI && defined(__WXMSW__)

#if wxUSE_UNICODE && !defined(__WXWINCE__)
    #define NEED_UNICODE_CHECK
#endif

#ifdef NEED_UNICODE_CHECK

// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner - helper
// This function changes 0x-values after a '\' (0x005c) to an unicode-value. This allows the input of
// unicode values with compilers that do not support the '\u'-method.
#if defined( __BORLANDC__ ) || defined( __WATCOMC__ )
int _stringescapes_to_wxchar( wxString* str )
{
  if ( str->find( wxChar('\\') ) == wxString::npos )
  {
    return 0;
  }

  unsigned int uiPos = 0;
  int iChanged = 0;
  while( uiPos+4 < str->size() )
  {
    uiPos = str->find( wxChar('\\'), uiPos );

    if (    uiPos == wxString::npos
         || uiPos+5 > str->size()
       )
    {
      return iChanged;
    }
    unsigned int uiValue = 0;
    for ( unsigned int ui = uiPos+1; ui < uiPos+5; ui++ )
    {
      uiValue <<= 4; // *16
      unsigned int uiChar = str->GetChar( ui );
      if (    uiChar >= 0x0030 // 0
           && uiChar <= 0x0039 // 9
         )
      {
        uiValue += uiChar-0x0030;
      }
      else if (    uiChar >= 0x0041 // A
                && uiChar <= 0x0046 // F
              )
      {
        uiValue += 10 + uiChar-0x0041;
      }
      else if (    uiChar >= 0x0061 // a
                && uiChar <= 0x0066 // f
              )
      {
        uiValue += 10 + uiChar-0x0061;
      }
      else
      {
        uiValue = 0;
        break;
      }
    }
    if ( uiValue )
    {
      wxString strSubstitute( wxT("1"));
      strSubstitute.SetChar(0, (wxChar)uiValue );
      str->replace( uiPos, 5, strSubstitute );
      ++iChanged;
    }
    ++uiPos;
  }
  return iChanged;
}
#endif // #if defined( __BORLANDC__ ) || __WATCOMC__
// new code end ---------------------------------------------------------------------------------------------

// check whether Unicode is available
static bool wxIsUnicodeAvailable()
{
    static const wchar_t *ERROR_STRING = L"wxWidgets Fatal Error";

    if ( wxGetOsVersion() != wxOS_WINDOWS_NT )
    {
        // we need to be built with MSLU support
#if !wxUSE_UNICODE_MSLU
        // note that we can use MessageBoxW() as it's implemented even under
        // Win9x - OTOH, we can't use wxGetTranslation() because the file APIs
        // used by wxLocale are not
/* old code start -------------------------------------------------------------------------------------------
        ::MessageBox
        (
         NULL,
         L"This program uses Unicode and requires Windows NT/2000/XP.\n"
         L"\n"
         L"Program aborted.",
         ERROR_STRING,
         MB_ICONERROR | MB_OK
        );
-- old code end / new code start ------------------------------------------------------------------------- */
// Modification by Christian Sturmlechner - new code only usable for TotoCalculator 2
        #if ( wxUSE_COMPILE_FOR_TOTOCALCULATOR )
        static const wchar_t *ERROR_MESSAGE;
        int iSystemLanguage = wxLocale::GetSystemLanguage();
        if (    iSystemLanguage >= wxLANGUAGE_GERMAN
             && iSystemLanguage <= wxLANGUAGE_GERMAN_SWISS
           )
        {
            ERROR_MESSAGE =
                L"This program uses Unicode and requires Windows NT/2000/XP/Vista/7/8/10.\n"
                L"Program aborted. Please get a version for\n"
                L"Windows 95, 98 and ME on www.totocalculator.com\n\n"
                L"Dieses Programm verwendet Unicode und braucht Windows NT/2000/XP/Vista/7/8/10.\n"
                L"Programm abgebrochen. Eine Version fuer\n"
                L"Windows 95, 98 und ME bekommen Sie auf www.totocalculator.com.";
        }
        else if (    iSystemLanguage >= wxLANGUAGE_FRENCH
                  && iSystemLanguage <= wxLANGUAGE_FRENCH_SWISS
           )
        {
            ERROR_MESSAGE =
                L"This program uses Unicode and requires Windows NT/2000/XP/Vista/7/8/10.\n"
                L"Program aborted. Please get a version for\n"
                L"Windows 95, 98 and ME on www.totocalculator.com\n\n"
                L"Ce programme utilise l'Unicode et un de ces OS est necessaire:\n"
                L"Windows NT/2000/XP/Vista/7/8/10.\n"
                L"Abandon du Programme. Il faut une version de\n"
                L"Windows 95, 98 et ME sur www.totocalculator.com";
        }
        else if (    iSystemLanguage >= wxLANGUAGE_ITALIAN
                  && iSystemLanguage <= wxLANGUAGE_ITALIAN_SWISS
           )
        {
            ERROR_MESSAGE =
                L"This program uses Unicode and requires Windows NT/2000/XP/Vista/7/8/10.\n"
                L"Program aborted. Please get a version for\n"
                L"Windows 95, 98 and ME on www.totocalculator.com\n\n"
                L"Questo programma usa Unicode e richiede Windows NT/2000/XP/Vista/7/8/10.\n"
                L"Esecuzione interrotta. Per favore scarica una versione per\n"
                L"Windows 95, 98 and ME da www.totocalculator.com.";
        }
        else if (    iSystemLanguage >= wxLANGUAGE_DUTCH
                  && iSystemLanguage <= wxLANGUAGE_DUTCH_BELGIAN
           )
        {
            ERROR_MESSAGE =
                L"This program uses Unicode and requires Windows NT/2000/XP/Vista/7/8/10.\n"
                L"Program aborted. Please get a version for\n"
                L"Windows 95, 98 and ME on www.totocalculator.com\n\n"
                L"Dit programma gebruikt Unicode en werkt onder Windows NT/2000/XP/Vista/7/8/10.\n"
                L"Programma beeindigd. De versie voor\n"
                L"Windows 95,98 en ME is te verkrijgen op www.totocalculator.com.";
        }
        else if (    iSystemLanguage >= wxLANGUAGE_SWEDISH
                  && iSystemLanguage <= wxLANGUAGE_SWEDISH_FINLAND
           )
        {
            // To be on the save side, I removed all umlauts from the Swedish translation.
            //    L"Det h*ae*r programmet anv*ae*nder Unicode och kr*ae*ver Windows NT/2000/XP/Vista/7/8/10.\n"
            //    L"Programmet avslutades. Var v*ae*nlig h*ae*mta en version f*oe*r\n"
            //    L"Windows 95, 98 and ME p*a with a little circle above* www.totocalculator.com.";
            ERROR_MESSAGE =
                L"This program uses Unicode and requires Windows NT/2000/XP/Vista/7/8/10.\n"
                L"Program aborted. Please get a version for\n"
                L"Windows 95, 98 and ME on www.totocalculator.com\n\n"
                L"Det har programmet anvander Unicode och kraver Windows NT/2000/XP/Vista/7/8/10.\n"
                L"Programmet avslutades. Var vanlig hamta en version for\n"
                L"Windows 95, 98 and ME pa www.totocalculator.com.";
        }
        else if (    iSystemLanguage >= wxLANGUAGE_PORTUGUESE
                  && iSystemLanguage <= wxLANGUAGE_PORTUGUESE_BRAZILIAN
           )
        {
            // To be on the save side, I removed all umlauts from the Portuguese translation.
            // vers*o -> versao
            ERROR_MESSAGE =
                L"This program uses Unicode and requires Windows NT/2000/XP/Vista/7/8/10.\n"
                L"Program aborted. Please get a version for\n"
                L"Windows 95, 98 and ME on www.totocalculator.com\n\n"
                L"Este programa usa Unicode e requer Windows NT/2000/XP/Vista/7/8/10.\n"
                L"Programa encerrou. Por favor obtenha uma versao para\n"
                L"Windows 95, 98 e ME em www.totocalculator.com.";
        }
        else if (    iSystemLanguage >= wxLANGUAGE_RUSSIAN
                  && iSystemLanguage <= wxLANGUAGE_RUSSIAN_UKRAINE
           )
        {
            #if defined( __BORLANDC__ )
            wxString strErr( wxT("This program uses Unicode and requires Windows NT/2000/XP/Vista/7/8/10.\nProgram aborted. Please get a version for\nWindows 95, 98 and ME on www.totocalculator.com\n\n\\042d\\0442\\0430 \\043f\\0440\\043e\\0433\\0440\\0430\\043c\\043c\\0430 \\0438\\0441\\043f\\043e\\043b\\044c\\0437\\0443\\0435\\0442 \\043a\\043e\\0434\\0438\\0440\\043e\\0432\\043a\\0443 Unicode \\0438 \\043d\\0443\\0436\\0434\\0430\\0435\\0442\\0441\\044f \\0432 Windows NT/2000/XP/Vista/7/8/10.\n\\041f\\0440\\043e\\0433\\0440\\0430\\043c\\043c\\0430 \\0437\\0430\\0432\\0435\\0440\\0448\\0430\\0435\\0442 \\0440\\0430\\0431\\043e\\0442\\0443. \\041f\\043e\\0436\\0430\\043b\\0443\\0439\\0441\\0442\\0430, \\043f\\043e\\043b\\0443\\0447\\0438\\0442\\0435 \\0432\\0435\\0440\\0441\\0438\\044e \\0434\\043b\\044f\nWindows 95, 98 \\0438 ME \\043d\\0430 \\0441\\0430\\0439\\0442\\0435 www.totocalculator.com") );
            _stringescapes_to_wxchar( &strErr );
            #elif defined( __WATCOMC__ )
            wxString strErr( wxT("This program uses Unicode and requires Windows NT/2000/XP/Vista/7/8/10.\nProgram aborted. Please get a version for\nWindows 95, 98 and ME on www.totocalculator.com\n\n") );
            strErr.append( wxT("\\042d\\0442\\0430 \\043f\\0440\\043e\\0433\\0440\\0430\\043c\\043c\\0430 \\0438\\0441\\043f\\043e\\043b\\044c\\0437\\0443\\0435\\0442 \\043a\\043e\\0434\\0438\\0440\\043e\\0432\\043a\\0443 Unicode \\0438 \\043d\\0443\\0436\\0434\\0430\\0435\\0442\\0441\\044f \\0432 Windows NT/2000/XP/Vista/7/8/10.\n\\041f\\0440\\043e\\0433\\0440\\0430") );
            strErr.append( wxT("\\043c\\043c\\0430 \\0437\\0430\\0432\\0435\\0440\\0448\\0430\\0435\\0442 \\0440\\0430\\0431\\043e\\0442\\0443. \\041f\\043e\\0436\\0430\\043b\\0443\\0439\\0441\\0442\\0430, \\043f\\043e\\043b\\0443\\0447\\0438\\0442\\0435 \\0432\\0435\\0440\\0441\\0438\\044e \\0434\\043b\\044f\nWindows 95, 98 \\0438 ME \\043d\\0430 \\0441\\0430\\0439\\0442\\0435 www.totocalculator.com") );
            _stringescapes_to_wxchar( &strErr );
            #else
            wxString strErr( wxT("This program uses Unicode and requires Windows NT/2000/XP/Vista/7/8/10.\nProgram aborted. Please get a version for\nWindows 95, 98 and ME on www.totocalculator.com\n\n\u042d\u0442\u0430 \u043f\u0440\u043e\u0433\u0440\u0430\u043c\u043c\u0430 \u0438\u0441\u043f\u043e\u043b\u044c\u0437\u0443\u0435\u0442 \u043a\u043e\u0434\u0438\u0440\u043e\u0432\u043a\u0443 Unicode \u0438 \u043d\u0443\u0436\u0434\u0430\u0435\u0442\u0441\u044f \u0432 Windows NT/2000/XP/Vista/7/8/10.\n\u041f\u0440\u043e\u0433\u0440\u0430\u043c\u043c\u0430 \u0437\u0430\u0432\u0435\u0440\u0448\u0430\u0435\u0442 \u0440\u0430\u0431\u043e\u0442\u0443. \u041f\u043e\u0436\u0430\u043b\u0443\u0439\u0441\u0442\u0430, \u043f\u043e\u043b\u0443\u0447\u0438\u0442\u0435 \u0432\u0435\u0440\u0441\u0438\u044e \u0434\u043b\u044f\nWindows 95, 98 \u0438 ME \u043d\u0430 \u0441\u0430\u0439\u0442\u0435 www.totocalculator.com") );
            #endif
            ERROR_MESSAGE = strErr.c_str();
        }
        else if ( iSystemLanguage >= wxLANGUAGE_HUNGARIAN )
        {
            #if defined( __BORLANDC__ )
            wxString strErr( wxT("This program uses Unicode and requires Windows NT/2000/XP/Vista/7/8/10.\nProgram aborted. Please get a version for\nWindows 95, 98 and ME on www.totocalculator.com\n\nEz a program Unicod-ot hasz\\00e1l. Aj\\00e1nlott Windows NT/2000/XP/Vista/7/8/10.\nK\\00e9rem, keresse a verzi\\00f3t! rendszerekre. A program megszakad.\nK\\00e9rj\\00fck l\\00e1togasson a honlapra: www.totocalculator.com") );
            _stringescapes_to_wxchar( &strErr );
            #elif defined( __WATCOMC__ )
            wxString strErr( wxT("This program uses Unicode and requires Windows NT/2000/XP/Vista/7/8/10.\nProgram aborted. Please get a version for\nWindows 95, 98 and ME on www.totocalculator.com\n\n") );
            strErr.append( wxT("Ez a program Unicod-ot hasz\\00e1l. Aj\\00e1nlott Windows NT/2000/XP/Vista/7/8/10.\nK\\00e9rem, keresse a verzi\\00f3t! rendszerekre. A program megszakad.\nK\\00e9rj\\00fck l\\00e1togasson a honlapra: www.totocalculator.com") );
            _stringescapes_to_wxchar( &strErr );
            #else
            wxString strErr( wxT("This program uses Unicode and requires Windows NT/2000/XP/Vista/7/8/10.\nProgram aborted. Please get a version for\nWindows 95, 98 and ME on www.totocalculator.com\n\nEz a program Unicod-ot hasz\u00e1l. Aj\u00e1nlott Windows NT/2000/XP/Vista/7/8/10.\nK\u00e9rem, keresse a verzi\u00f3t! rendszerekre. A program megszakad.\nK\u00e9rj\u00fck l\u00e1togasson a honlapra: www.totocalculator.com") );
            #endif
            ERROR_MESSAGE = strErr.c_str();
        }
        else // all other: English
        {
            ERROR_MESSAGE =
                L"This program uses Unicode and requires Windows NT/2000/XP/Vista/7/8/10.\n"
                L"Program aborted. Please get a version for\n"
                L"Windows 95, 98 and ME on www.totocalculator.com";
        }
        ::MessageBox( NULL, ERROR_MESSAGE, ERROR_STRING, MB_ICONERROR | MB_OK  );
        #else // !wxUSE_COMPILE_FOR_TOTOCALCULATOR
        ::MessageBox
        (
         NULL,
         L"This program uses Unicode and requires Windows NT/2000/XP/Vista/7/8/10.\n"
         L"\n"
         L"Program aborted.",
         ERROR_STRING,
         MB_ICONERROR | MB_OK
        );
        #endif // wxUSE_COMPILE_FOR_TOTOCALCULATOR / !wxUSE_COMPILE_FOR_TOTOCALCULATOR
// new code end ---------------------------------------------------------------------------------------------

        return false;
#else // wxUSE_UNICODE_MSLU
        // and the MSLU DLL must also be available
        HMODULE hmod = ::LoadLibraryA("unicows.dll");
        if ( !hmod )
        {
            ::MessageBox
            (
             NULL,
             L"This program uses Unicode and requires unicows.dll to work "
             L"under current operating system.\n"
             L"\n"
             L"Please install unicows.dll and relaunch the program.",
             ERROR_STRING,
             MB_ICONERROR | MB_OK
            );
            return false;
        }

        // this is not really necessary but be tidy
        ::FreeLibrary(hmod);

        // finally do the last check: has unicows.lib initialized correctly?
        hmod = ::LoadLibraryW(L"unicows.dll");
        if ( !hmod )
        {
            ::MessageBox
            (
             NULL,
             L"This program uses Unicode but is not using unicows.dll\n"
             L"correctly and so cannot work under current operating system.\n"
             L"Please contact the program author for an updated version.\n"
             L"\n"
             L"Program aborted.",
             ERROR_STRING,
             MB_ICONERROR | MB_OK
            );

            return false;
        }

        ::FreeLibrary(hmod);
#endif // !wxUSE_UNICODE_MSLU
    }

    return true;
}

#endif // NEED_UNICODE_CHECK

// ----------------------------------------------------------------------------
// Windows-specific wxEntry
// ----------------------------------------------------------------------------

// helper function used to clean up in wxEntry() just below
//
// notice that argv elements are supposed to be allocated using malloc() while
// argv array itself is allocated with new
static void wxFreeArgs(int argc, wxChar **argv)
{
    for ( int i = 0; i < argc; i++ )
    {
        free(argv[i]);
    }

    delete [] argv;
}

WXDLLEXPORT int wxEntry(HINSTANCE hInstance,
                        HINSTANCE WXUNUSED(hPrevInstance),
                        wxCmdLineArgType WXUNUSED(pCmdLine),
                        int nCmdShow)
{
    // the first thing to do is to check if we're trying to run an Unicode
    // program under Win9x w/o MSLU emulation layer - if so, abort right now
    // as it has no chance to work and has all chances to crash
#ifdef NEED_UNICODE_CHECK
    if ( !wxIsUnicodeAvailable() )
        return -1;
#endif // NEED_UNICODE_CHECK


    // remember the parameters Windows gave us
    wxSetInstance(hInstance);
    wxApp::m_nCmdShow = nCmdShow;

    // parse the command line: we can't use pCmdLine in Unicode build so it is
    // simpler to never use it at all (this also results in a more correct
    // argv[0])

    // break the command line in words
    wxArrayString args;

    const wxChar *cmdLine = ::GetCommandLine();
    if ( cmdLine )
    {
        args = wxCmdLineParser::ConvertStringToArgs(cmdLine);
    }

#ifdef __WXWINCE__
    // WinCE doesn't insert the program itself, so do it ourselves.
    args.Insert(wxGetFullModuleName(), 0);
#endif

    int argc = args.GetCount();

    // +1 here for the terminating NULL
    wxChar **argv = new wxChar *[argc + 1];
    for ( int i = 0; i < argc; i++ )
    {
        argv[i] = wxStrdup(args[i]);
    }

    // argv[] must be NULL-terminated
    argv[argc] = NULL;

// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner - avoid runtime error with digital mars
    #if !defined( __DMC__ )
// new code end ---------------------------------------------------------------------------------------------
    wxON_BLOCK_EXIT2(wxFreeArgs, argc, argv);
// new code start -------------------------------------------------------------------------------------------
// Modification by Christian Sturmlechner
    #endif // !__DMC__
// new code end ---------------------------------------------------------------------------------------------

    return wxEntry(argc, argv);
}

#endif // wxUSE_GUI && __WXMSW__

// ----------------------------------------------------------------------------
// global HINSTANCE
// ----------------------------------------------------------------------------

#if wxUSE_BASE

HINSTANCE wxhInstance = 0;

extern "C" HINSTANCE wxGetInstance()
{
    return wxhInstance;
}

void wxSetInstance(HINSTANCE hInst)
{
    wxhInstance = hInst;
}

#endif // wxUSE_BASE
